#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: mso_tenant
short_description: Manage tenants
description:
- Manage tenants on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
version_added: '2.8'
options:
  tenant:
    description:
    - The name of the tenant.
    type: str
    required: yes
    aliases: [ name ]
  display_name:
    description:
    - The name of the tenant to be displayed in the web UI.
    type: str
    required: yes
  description:
    description:
    - The description for this tenant.
    type: str
  users:
    description:
    - A list of associated users for this tenant.
    - Using this property will replace any existing associated users.
    type: list
  sites:
    description:
    - A list of associated sites for this tenant.
    - Using this property will replace any existing associated sites.
    type: list
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment: mso
'''

EXAMPLES = r'''
- name: Add a new tenant
  mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: north_europe
    display_name: North European Datacenter
    description: This tenant manages the NEDC environment.
    state: present
  delegate_to: localhost

- name: Remove a tenant
  mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: north_europe
    state: absent
  delegate_to: localhost

- name: Query a tenant
  mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: north_europe
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all tenants
  mso_tenant:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.aci.mso import MSOModule, mso_argument_spec, issubset


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        description=dict(type='str'),
        display_name=dict(type='str'),
        tenant=dict(type='str', aliases=['name']),
        users=dict(type='list'),
        sites=dict(type='list'),
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['tenant']],
            ['state', 'present', ['tenant']],
        ],
    )

    description = module.params['description']
    display_name = module.params['display_name']
    tenant = module.params['tenant']
    state = module.params['state']

    mso = MSOModule(module)

    # Convert sites and users
    sites = mso.lookup_sites(module.params['sites'])
    users = mso.lookup_users(module.params['users'])

    tenant_id = None
    path = 'tenants'

    # Query for existing object(s)
    if tenant:
        mso.existing = mso.get_obj(path, name=tenant)
        if mso.existing:
            tenant_id = mso.existing['id']
            # If we found an existing object, continue with it
            path = 'tenants/{id}'.format(id=tenant_id)
    else:
        mso.existing = mso.query_objs(path)

    if state == 'query':
        pass

    elif state == 'absent':
        mso.previous = mso.existing
        if mso.existing:
            if module.check_mode:
                mso.existing = {}
            else:
                mso.existing = mso.request(path, method='DELETE')

    elif state == 'present':
        mso.previous = mso.existing

        payload = dict(
            description=description,
            id=tenant_id,
            name=tenant,
            displayName=display_name,
            siteAssociations=sites,
            userAssociations=users,
        )

        mso.sanitize(payload, collate=True)

        # Ensure displayName is not undefined
        if mso.sent.get('displayName') is None:
            mso.sent['displayName'] = tenant

        # Ensure tenant has at least admin user
        if mso.sent.get('userAssociations') is None:
            mso.sent['userAssociations'] = [dict(userId="0000ffff0000000000000020")]

        if mso.existing:
            if not issubset(mso.sent, mso.existing):
                if module.check_mode:
                    mso.existing = mso.proposed
                else:
                    mso.existing = mso.request(path, method='PUT', data=mso.sent)
        else:
            if module.check_mode:
                mso.existing = mso.proposed
            else:
                mso.existing = mso.request(path, method='POST', data=mso.sent)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_dnsfilter_profile
short_description: Configure DNS domain filter profiles in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure dnsfilter feature and profile category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    dnsfilter_profile:
        description:
            - Configure DNS domain filter profiles.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            block-action:
                description:
                    - Action to take for blocked domains.
                choices:
                    - block
                    - redirect
            block-botnet:
                description:
                    - Enable/disable blocking botnet C&C DNS lookups.
                choices:
                    - disable
                    - enable
            comment:
                description:
                    - Comment.
            domain-filter:
                description:
                    - Domain filter settings.
                suboptions:
                    domain-filter-table:
                        description:
                            - DNS domain filter table ID. Source dnsfilter.domain-filter.id.
            external-ip-blocklist:
                description:
                    - One or more external IP block lists.
                suboptions:
                    name:
                        description:
                            - External domain block list name. Source system.external-resource.name.
                        required: true
            ftgd-dns:
                description:
                    - FortiGuard DNS Filter settings.
                suboptions:
                    filters:
                        description:
                            - FortiGuard DNS domain filters.
                        suboptions:
                            action:
                                description:
                                    - Action to take for DNS requests matching the category.
                                choices:
                                    - block
                                    - monitor
                            category:
                                description:
                                    - Category number.
                            id:
                                description:
                                    - ID number.
                                required: true
                            log:
                                description:
                                    - Enable/disable DNS filter logging for this DNS profile.
                                choices:
                                    - enable
                                    - disable
                    options:
                        description:
                            - FortiGuard DNS filter options.
                        choices:
                            - error-allow
                            - ftgd-disable
            log-all-domain:
                description:
                    - Enable/disable logging of all domains visited (detailed DNS logging).
                choices:
                    - enable
                    - disable
            name:
                description:
                    - Profile name.
                required: true
            redirect-portal:
                description:
                    - IP address of the SDNS redirect portal.
            safe-search:
                description:
                    - Enable/disable Google, Bing, and YouTube safe search.
                choices:
                    - disable
                    - enable
            sdns-domain-log:
                description:
                    - Enable/disable domain filtering and botnet domain logging.
                choices:
                    - enable
                    - disable
            sdns-ftgd-err-log:
                description:
                    - Enable/disable FortiGuard SDNS rating error logging.
                choices:
                    - enable
                    - disable
            youtube-restrict:
                description:
                    - Set safe search for YouTube restriction level.
                choices:
                    - strict
                    - moderate
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure DNS domain filter profiles.
    fortios_dnsfilter_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      dnsfilter_profile:
        state: "present"
        block-action: "block"
        block-botnet: "disable"
        comment: "Comment."
        domain-filter:
            domain-filter-table: "7 (source dnsfilter.domain-filter.id)"
        external-ip-blocklist:
         -
            name: "default_name_9 (source system.external-resource.name)"
        ftgd-dns:
            filters:
             -
                action: "block"
                category: "13"
                id:  "14"
                log: "enable"
            options: "error-allow"
        log-all-domain: "enable"
        name: "default_name_18"
        redirect-portal: "<your_own_value>"
        safe-search: "disable"
        sdns-domain-log: "enable"
        sdns-ftgd-err-log: "enable"
        youtube-restrict: "strict"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_dnsfilter_profile_data(json):
    option_list = ['block-action', 'block-botnet', 'comment',
                   'domain-filter', 'external-ip-blocklist', 'ftgd-dns',
                   'log-all-domain', 'name', 'redirect-portal',
                   'safe-search', 'sdns-domain-log', 'sdns-ftgd-err-log',
                   'youtube-restrict']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def dnsfilter_profile(data, fos):
    vdom = data['vdom']
    dnsfilter_profile_data = data['dnsfilter_profile']
    filtered_data = filter_dnsfilter_profile_data(dnsfilter_profile_data)
    if dnsfilter_profile_data['state'] == "present":
        return fos.set('dnsfilter',
                       'profile',
                       data=filtered_data,
                       vdom=vdom)

    elif dnsfilter_profile_data['state'] == "absent":
        return fos.delete('dnsfilter',
                          'profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_dnsfilter(data, fos):
    login(data)

    methodlist = ['dnsfilter_profile']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "dnsfilter_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "block-action": {"required": False, "type": "str",
                                 "choices": ["block", "redirect"]},
                "block-botnet": {"required": False, "type": "str",
                                 "choices": ["disable", "enable"]},
                "comment": {"required": False, "type": "str"},
                "domain-filter": {"required": False, "type": "dict",
                                  "options": {
                                      "domain-filter-table": {"required": False, "type": "int"}
                                  }},
                "external-ip-blocklist": {"required": False, "type": "list",
                                          "options": {
                                              "name": {"required": True, "type": "str"}
                                          }},
                "ftgd-dns": {"required": False, "type": "dict",
                             "options": {
                                 "filters": {"required": False, "type": "list",
                                             "options": {
                                                 "action": {"required": False, "type": "str",
                                                            "choices": ["block", "monitor"]},
                                                 "category": {"required": False, "type": "int"},
                                                 "id": {"required": True, "type": "int"},
                                                 "log": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]}
                                             }},
                                 "options": {"required": False, "type": "str",
                                             "choices": ["error-allow", "ftgd-disable"]}
                             }},
                "log-all-domain": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "name": {"required": True, "type": "str"},
                "redirect-portal": {"required": False, "type": "str"},
                "safe-search": {"required": False, "type": "str",
                                "choices": ["disable", "enable"]},
                "sdns-domain-log": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "sdns-ftgd-err-log": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "youtube-restrict": {"required": False, "type": "str",
                                     "choices": ["strict", "moderate"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_dnsfilter(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Google
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file at
#     https://www.github.com/GoogleCloudPlatform/magic-modules
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function

__metaclass__ = type

################################################################################
# Documentation
################################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ["preview"], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: gcp_sql_instance
description:
- Represents a Cloud SQL instance. Cloud SQL instances are SQL databases hosted in
  Google's cloud. The Instances resource provides methods for common configuration
  and management tasks.
short_description: Creates a GCP Instance
version_added: 2.7
author: Google Inc. (@googlecloudplatform)
requirements:
- python >= 2.6
- requests >= 2.18.4
- google-auth >= 1.3.0
options:
  state:
    description:
    - Whether the given object should exist in GCP
    choices:
    - present
    - absent
    default: present
  backend_type:
    description:
    - "* FIRST_GEN: First Generation instance. MySQL only."
    - "* SECOND_GEN: Second Generation instance or PostgreSQL instance."
    - "* EXTERNAL: A database server that is not managed by Google."
    required: false
    choices:
    - FIRST_GEN
    - SECOND_GEN
    - EXTERNAL
  connection_name:
    description:
    - Connection name of the Cloud SQL instance used in connection strings.
    required: false
  database_version:
    description:
    - The database engine type and version. For First Generation instances, can be
      MYSQL_5_5, or MYSQL_5_6. For Second Generation instances, can be MYSQL_5_6 or
      MYSQL_5_7. Defaults to MYSQL_5_6.
    - 'PostgreSQL instances: POSTGRES_9_6 The databaseVersion property can not be
      changed after instance creation.'
    required: false
    choices:
    - MYSQL_5_5
    - MYSQL_5_6
    - MYSQL_5_7
    - POSTGRES_9_6
  failover_replica:
    description:
    - The name and status of the failover replica. This property is applicable only
      to Second Generation instances.
    required: false
    suboptions:
      name:
        description:
        - The name of the failover replica. If specified at instance creation, a failover
          replica is created for the instance. The name doesn't include the project
          ID. This property is applicable only to Second Generation instances.
        required: false
  instance_type:
    description:
    - The instance type. This can be one of the following.
    - "* CLOUD_SQL_INSTANCE: A Cloud SQL instance that is not replicating from a master."
    - "* ON_PREMISES_INSTANCE: An instance running on the customer's premises."
    - "* READ_REPLICA_INSTANCE: A Cloud SQL instance configured as a read-replica."
    required: false
    choices:
    - CLOUD_SQL_INSTANCE
    - ON_PREMISES_INSTANCE
    - READ_REPLICA_INSTANCE
  ipv6_address:
    description:
    - The IPv6 address assigned to the instance. This property is applicable only
      to First Generation instances.
    required: false
  master_instance_name:
    description:
    - The name of the instance which will act as master in the replication setup.
    required: false
  max_disk_size:
    description:
    - The maximum disk size of the instance in bytes.
    required: false
  name:
    description:
    - Name of the Cloud SQL instance. This does not include the project ID.
    required: true
  region:
    description:
    - The geographical region. Defaults to us-central or us-central1 depending on
      the instance type (First Generation or Second Generation/PostgreSQL).
    required: false
  replica_configuration:
    description:
    - Configuration specific to failover replicas and read replicas.
    required: false
    suboptions:
      failover_target:
        description:
        - Specifies if the replica is the failover target. If the field is set to
          true the replica will be designated as a failover replica.
        - In case the master instance fails, the replica instance will be promoted
          as the new master instance.
        - Only one replica can be specified as failover target, and the replica has
          to be in different zone with the master instance.
        required: false
        type: bool
      mysql_replica_configuration:
        description:
        - MySQL specific configuration when replicating from a MySQL on-premises master.
          Replication configuration information such as the username, password, certificates,
          and keys are not stored in the instance metadata. The configuration information
          is used only to set up the replication connection and is stored by MySQL
          in a file named master.info in the data directory.
        required: false
        suboptions:
          ca_certificate:
            description:
            - PEM representation of the trusted CA's x509 certificate.
            required: false
          client_certificate:
            description:
            - PEM representation of the slave's x509 certificate .
            required: false
          client_key:
            description:
            - PEM representation of the slave's private key. The corresponding public
              key is encoded in the client's certificate.
            required: false
          connect_retry_interval:
            description:
            - Seconds to wait between connect retries. MySQL's default is 60 seconds.
            required: false
          dump_file_path:
            description:
            - Path to a SQL dump file in Google Cloud Storage from which the slave
              instance is to be created. The URI is in the form gs://bucketName/fileName.
              Compressed gzip files (.gz) are also supported. Dumps should have the
              binlog co-ordinates from which replication should begin. This can be
              accomplished by setting --master-data to 1 when using mysqldump.
            required: false
          master_heartbeat_period:
            description:
            - Interval in milliseconds between replication heartbeats.
            required: false
          password:
            description:
            - The password for the replication connection.
            required: false
          ssl_cipher:
            description:
            - A list of permissible ciphers to use for SSL encryption.
            required: false
          username:
            description:
            - The username for the replication connection.
            required: false
          verify_server_certificate:
            description:
            - Whether or not to check the master's Common Name value in the certificate
              that it sends during the SSL handshake.
            required: false
            type: bool
      replica_names:
        description:
        - The replicas of the instance.
        required: false
      service_account_email_address:
        description:
        - The service account email address assigned to the instance. This property
          is applicable only to Second Generation instances.
        required: false
  settings:
    description:
    - The user settings.
    required: false
    suboptions:
      ip_configuration:
        description:
        - The settings for IP Management. This allows to enable or disable the instance
          IP and manage which external networks can connect to the instance. The IPv4
          address cannot be disabled for Second Generation instances.
        required: false
        suboptions:
          ipv4_enabled:
            description:
            - Whether the instance should be assigned an IP address or not.
            required: false
            type: bool
          authorized_networks:
            description:
            - The list of external networks that are allowed to connect to the instance
              using the IP. In CIDR notation, also known as 'slash' notation (e.g.
              192.168.100.0/24).
            required: false
            suboptions:
              expiration_time:
                description:
                - The time when this access control entry expires in RFC 3339 format,
                  for example 2012-11-15T16:19:00.094Z.
                required: false
              name:
                description:
                - An optional label to identify this entry.
                required: false
              value:
                description:
                - The whitelisted value for the access control list. For example,
                  to grant access to a client from an external IP (IPv4 or IPv6) address
                  or subnet, use that address or subnet here.
                required: false
          require_ssl:
            description:
            - Whether the mysqld should default to 'REQUIRE X509' for users connecting
              over IP.
            required: false
            type: bool
      tier:
        description:
        - The tier or machine type for this instance, for example db-n1-standard-1.
          For MySQL instances, this field determines whether the instance is Second
          Generation (recommended) or First Generation.
        required: false
      availability_type:
        description:
        - The availabilityType define if your postgres instance is run zonal or regional.
        required: false
        choices:
        - ZONAL
        - REGIONAL
      backup_configuration:
        description:
        - The daily backup configuration for the instance.
        required: false
        suboptions:
          enabled:
            description:
            - Enable Autobackup for your instance.
            required: false
            type: bool
          binary_log_enabled:
            description:
            - Whether binary log is enabled. If backup configuration is disabled,
              binary log must be disabled as well. MySQL only.
            required: false
            type: bool
          start_time:
            description:
            - Define the backup start time in UTC (HH:MM) .
            required: false
extends_documentation_fragment: gcp
'''

EXAMPLES = '''
- name: create a instance
  gcp_sql_instance:
    name: "{{resource_name}}-2"
    settings:
      ip_configuration:
        authorized_networks:
        - name: google dns server
          value: 8.8.8.8/32
      tier: db-n1-standard-1
    region: us-central1
    project: test_project
    auth_kind: serviceaccount
    service_account_file: "/tmp/auth.pem"
    state: present
'''

RETURN = '''
backendType:
  description:
  - "* FIRST_GEN: First Generation instance. MySQL only."
  - "* SECOND_GEN: Second Generation instance or PostgreSQL instance."
  - "* EXTERNAL: A database server that is not managed by Google."
  returned: success
  type: str
connectionName:
  description:
  - Connection name of the Cloud SQL instance used in connection strings.
  returned: success
  type: str
databaseVersion:
  description:
  - The database engine type and version. For First Generation instances, can be MYSQL_5_5,
    or MYSQL_5_6. For Second Generation instances, can be MYSQL_5_6 or MYSQL_5_7.
    Defaults to MYSQL_5_6.
  - 'PostgreSQL instances: POSTGRES_9_6 The databaseVersion property can not be changed
    after instance creation.'
  returned: success
  type: str
failoverReplica:
  description:
  - The name and status of the failover replica. This property is applicable only
    to Second Generation instances.
  returned: success
  type: complex
  contains:
    available:
      description:
      - The availability status of the failover replica. A false status indicates
        that the failover replica is out of sync. The master can only failover to
        the failover replica when the status is true.
      returned: success
      type: bool
    name:
      description:
      - The name of the failover replica. If specified at instance creation, a failover
        replica is created for the instance. The name doesn't include the project
        ID. This property is applicable only to Second Generation instances.
      returned: success
      type: str
instanceType:
  description:
  - The instance type. This can be one of the following.
  - "* CLOUD_SQL_INSTANCE: A Cloud SQL instance that is not replicating from a master."
  - "* ON_PREMISES_INSTANCE: An instance running on the customer's premises."
  - "* READ_REPLICA_INSTANCE: A Cloud SQL instance configured as a read-replica."
  returned: success
  type: str
ipAddresses:
  description:
  - The assigned IP addresses for the instance.
  returned: success
  type: complex
  contains:
    ipAddress:
      description:
      - The IP address assigned.
      returned: success
      type: str
    timeToRetire:
      description:
      - The due time for this IP to be retired in RFC 3339 format, for example 2012-11-15T16:19:00.094Z.
        This field is only available when the IP is scheduled to be retired.
      returned: success
      type: str
    type:
      description:
      - The type of this IP address. A PRIMARY address is an address that can accept
        incoming connections. An OUTGOING address is the source address of connections
        originating from the instance, if supported.
      returned: success
      type: str
ipv6Address:
  description:
  - The IPv6 address assigned to the instance. This property is applicable only to
    First Generation instances.
  returned: success
  type: str
masterInstanceName:
  description:
  - The name of the instance which will act as master in the replication setup.
  returned: success
  type: str
maxDiskSize:
  description:
  - The maximum disk size of the instance in bytes.
  returned: success
  type: int
name:
  description:
  - Name of the Cloud SQL instance. This does not include the project ID.
  returned: success
  type: str
region:
  description:
  - The geographical region. Defaults to us-central or us-central1 depending on the
    instance type (First Generation or Second Generation/PostgreSQL).
  returned: success
  type: str
replicaConfiguration:
  description:
  - Configuration specific to failover replicas and read replicas.
  returned: success
  type: complex
  contains:
    failoverTarget:
      description:
      - Specifies if the replica is the failover target. If the field is set to true
        the replica will be designated as a failover replica.
      - In case the master instance fails, the replica instance will be promoted as
        the new master instance.
      - Only one replica can be specified as failover target, and the replica has
        to be in different zone with the master instance.
      returned: success
      type: bool
    mysqlReplicaConfiguration:
      description:
      - MySQL specific configuration when replicating from a MySQL on-premises master.
        Replication configuration information such as the username, password, certificates,
        and keys are not stored in the instance metadata. The configuration information
        is used only to set up the replication connection and is stored by MySQL in
        a file named master.info in the data directory.
      returned: success
      type: complex
      contains:
        caCertificate:
          description:
          - PEM representation of the trusted CA's x509 certificate.
          returned: success
          type: str
        clientCertificate:
          description:
          - PEM representation of the slave's x509 certificate .
          returned: success
          type: str
        clientKey:
          description:
          - PEM representation of the slave's private key. The corresponding public
            key is encoded in the client's certificate.
          returned: success
          type: str
        connectRetryInterval:
          description:
          - Seconds to wait between connect retries. MySQL's default is 60 seconds.
          returned: success
          type: int
        dumpFilePath:
          description:
          - Path to a SQL dump file in Google Cloud Storage from which the slave instance
            is to be created. The URI is in the form gs://bucketName/fileName. Compressed
            gzip files (.gz) are also supported. Dumps should have the binlog co-ordinates
            from which replication should begin. This can be accomplished by setting
            --master-data to 1 when using mysqldump.
          returned: success
          type: str
        masterHeartbeatPeriod:
          description:
          - Interval in milliseconds between replication heartbeats.
          returned: success
          type: int
        password:
          description:
          - The password for the replication connection.
          returned: success
          type: str
        sslCipher:
          description:
          - A list of permissible ciphers to use for SSL encryption.
          returned: success
          type: str
        username:
          description:
          - The username for the replication connection.
          returned: success
          type: str
        verifyServerCertificate:
          description:
          - Whether or not to check the master's Common Name value in the certificate
            that it sends during the SSL handshake.
          returned: success
          type: bool
    replicaNames:
      description:
      - The replicas of the instance.
      returned: success
      type: list
    serviceAccountEmailAddress:
      description:
      - The service account email address assigned to the instance. This property
        is applicable only to Second Generation instances.
      returned: success
      type: str
settings:
  description:
  - The user settings.
  returned: success
  type: complex
  contains:
    ipConfiguration:
      description:
      - The settings for IP Management. This allows to enable or disable the instance
        IP and manage which external networks can connect to the instance. The IPv4
        address cannot be disabled for Second Generation instances.
      returned: success
      type: complex
      contains:
        ipv4Enabled:
          description:
          - Whether the instance should be assigned an IP address or not.
          returned: success
          type: bool
        authorizedNetworks:
          description:
          - The list of external networks that are allowed to connect to the instance
            using the IP. In CIDR notation, also known as 'slash' notation (e.g. 192.168.100.0/24).
          returned: success
          type: complex
          contains:
            expirationTime:
              description:
              - The time when this access control entry expires in RFC 3339 format,
                for example 2012-11-15T16:19:00.094Z.
              returned: success
              type: str
            name:
              description:
              - An optional label to identify this entry.
              returned: success
              type: str
            value:
              description:
              - The whitelisted value for the access control list. For example, to
                grant access to a client from an external IP (IPv4 or IPv6) address
                or subnet, use that address or subnet here.
              returned: success
              type: str
        requireSsl:
          description:
          - Whether the mysqld should default to 'REQUIRE X509' for users connecting
            over IP.
          returned: success
          type: bool
    tier:
      description:
      - The tier or machine type for this instance, for example db-n1-standard-1.
        For MySQL instances, this field determines whether the instance is Second
        Generation (recommended) or First Generation.
      returned: success
      type: str
    availabilityType:
      description:
      - The availabilityType define if your postgres instance is run zonal or regional.
      returned: success
      type: str
    backupConfiguration:
      description:
      - The daily backup configuration for the instance.
      returned: success
      type: complex
      contains:
        enabled:
          description:
          - Enable Autobackup for your instance.
          returned: success
          type: bool
        binaryLogEnabled:
          description:
          - Whether binary log is enabled. If backup configuration is disabled, binary
            log must be disabled as well. MySQL only.
          returned: success
          type: bool
        startTime:
          description:
          - Define the backup start time in UTC (HH:MM) .
          returned: success
          type: str
    settingsVersion:
      description:
      - The version of instance settings. This is a required field for update method
        to make sure concurrent updates are handled properly. During update, use the
        most recent settingsVersion value for this instance and do not try to update
        this value.
      returned: success
      type: int
'''

################################################################################
# Imports
################################################################################

from ansible.module_utils.gcp_utils import navigate_hash, GcpSession, GcpModule, GcpRequest, remove_nones_from_dict, replace_resource_dict
import json
import time

################################################################################
# Main
################################################################################


def main():
    """Main function"""

    module = GcpModule(
        argument_spec=dict(
            state=dict(default='present', choices=['present', 'absent'], type='str'),
            backend_type=dict(type='str', choices=['FIRST_GEN', 'SECOND_GEN', 'EXTERNAL']),
            connection_name=dict(type='str'),
            database_version=dict(type='str', choices=['MYSQL_5_5', 'MYSQL_5_6', 'MYSQL_5_7', 'POSTGRES_9_6']),
            failover_replica=dict(type='dict', options=dict(name=dict(type='str'))),
            instance_type=dict(type='str', choices=['CLOUD_SQL_INSTANCE', 'ON_PREMISES_INSTANCE', 'READ_REPLICA_INSTANCE']),
            ipv6_address=dict(type='str'),
            master_instance_name=dict(type='str'),
            max_disk_size=dict(type='int'),
            name=dict(required=True, type='str'),
            region=dict(type='str'),
            replica_configuration=dict(
                type='dict',
                options=dict(
                    failover_target=dict(type='bool'),
                    mysql_replica_configuration=dict(
                        type='dict',
                        options=dict(
                            ca_certificate=dict(type='str'),
                            client_certificate=dict(type='str'),
                            client_key=dict(type='str'),
                            connect_retry_interval=dict(type='int'),
                            dump_file_path=dict(type='str'),
                            master_heartbeat_period=dict(type='int'),
                            password=dict(type='str'),
                            ssl_cipher=dict(type='str'),
                            username=dict(type='str'),
                            verify_server_certificate=dict(type='bool'),
                        ),
                    ),
                    replica_names=dict(type='list', elements='str'),
                    service_account_email_address=dict(type='str'),
                ),
            ),
            settings=dict(
                type='dict',
                options=dict(
                    ip_configuration=dict(
                        type='dict',
                        options=dict(
                            ipv4_enabled=dict(type='bool'),
                            authorized_networks=dict(
                                type='list', elements='dict', options=dict(expiration_time=dict(type='str'), name=dict(type='str'), value=dict(type='str'))
                            ),
                            require_ssl=dict(type='bool'),
                        ),
                    ),
                    tier=dict(type='str'),
                    availability_type=dict(type='str', choices=['ZONAL', 'REGIONAL']),
                    backup_configuration=dict(
                        type='dict', options=dict(enabled=dict(type='bool'), binary_log_enabled=dict(type='bool'), start_time=dict(type='str'))
                    ),
                ),
            ),
        )
    )

    if not module.params['scopes']:
        module.params['scopes'] = ['https://www.googleapis.com/auth/sqlservice.admin']

    state = module.params['state']
    kind = 'sql#instance'

    fetch = fetch_resource(module, self_link(module), kind)
    changed = False

    if fetch:
        if state == 'present':
            if is_different(module, fetch):
                update(module, self_link(module), kind, fetch)
                fetch = fetch_resource(module, self_link(module), kind)
                changed = True
        else:
            delete(module, self_link(module), kind, fetch)
            fetch = {}
            changed = True
    else:
        if state == 'present':
            fetch = create(module, collection(module), kind)
            changed = True
        else:
            fetch = {}

    fetch.update({'changed': changed})

    module.exit_json(**fetch)


def create(module, link, kind):
    auth = GcpSession(module, 'sql')
    return wait_for_operation(module, auth.post(link, resource_to_request(module)))


def update(module, link, kind, fetch):
    auth = GcpSession(module, 'sql')
    return wait_for_operation(module, auth.put(link, resource_to_request(module)))


def delete(module, link, kind, fetch):
    auth = GcpSession(module, 'sql')
    return wait_for_operation(module, auth.delete(link))


def resource_to_request(module):
    request = {
        u'kind': 'sql#instance',
        u'backendType': module.params.get('backend_type'),
        u'connectionName': module.params.get('connection_name'),
        u'databaseVersion': module.params.get('database_version'),
        u'failoverReplica': InstanceFailoverreplica(module.params.get('failover_replica', {}), module).to_request(),
        u'instanceType': module.params.get('instance_type'),
        u'ipv6Address': module.params.get('ipv6_address'),
        u'masterInstanceName': module.params.get('master_instance_name'),
        u'maxDiskSize': module.params.get('max_disk_size'),
        u'name': module.params.get('name'),
        u'region': module.params.get('region'),
        u'replicaConfiguration': InstanceReplicaconfiguration(module.params.get('replica_configuration', {}), module).to_request(),
        u'settings': InstanceSettings(module.params.get('settings', {}), module).to_request(),
    }
    return_vals = {}
    for k, v in request.items():
        if v or v is False:
            return_vals[k] = v

    return return_vals


def fetch_resource(module, link, kind, allow_not_found=True):
    auth = GcpSession(module, 'sql')
    return return_if_object(module, auth.get(link), kind, allow_not_found)


def self_link(module):
    return "https://www.googleapis.com/sql/v1beta4/projects/{project}/instances/{name}".format(**module.params)


def collection(module):
    return "https://www.googleapis.com/sql/v1beta4/projects/{project}/instances".format(**module.params)


def return_if_object(module, response, kind, allow_not_found=False):
    # If not found, return nothing.
    if allow_not_found and response.status_code == 404:
        return None

    # If no content, return nothing.
    if response.status_code == 204:
        return None

    # SQL only: return on 403 if not exist
    if allow_not_found and response.status_code == 403:
        return None

    try:
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError) as inst:
        module.fail_json(msg="Invalid JSON response with error: %s" % inst)

    if navigate_hash(result, ['error', 'errors']):
        module.fail_json(msg=navigate_hash(result, ['error', 'errors']))

    return result


def is_different(module, response):
    request = resource_to_request(module)
    response = response_to_hash(module, response)

    # Remove all output-only from response.
    response_vals = {}
    for k, v in response.items():
        if k in request:
            response_vals[k] = v

    request_vals = {}
    for k, v in request.items():
        if k in response:
            request_vals[k] = v

    return GcpRequest(request_vals) != GcpRequest(response_vals)


# Remove unnecessary properties from the response.
# This is for doing comparisons with Ansible's current parameters.
def response_to_hash(module, response):
    return {
        u'backendType': response.get(u'backendType'),
        u'connectionName': response.get(u'connectionName'),
        u'databaseVersion': response.get(u'databaseVersion'),
        u'failoverReplica': InstanceFailoverreplica(response.get(u'failoverReplica', {}), module).from_response(),
        u'instanceType': response.get(u'instanceType'),
        u'ipAddresses': InstanceIpaddressesArray(response.get(u'ipAddresses', []), module).from_response(),
        u'ipv6Address': response.get(u'ipv6Address'),
        u'masterInstanceName': response.get(u'masterInstanceName'),
        u'maxDiskSize': response.get(u'maxDiskSize'),
        u'name': response.get(u'name'),
        u'region': response.get(u'region'),
        u'replicaConfiguration': InstanceReplicaconfiguration(response.get(u'replicaConfiguration', {}), module).from_response(),
        u'settings': InstanceSettings(response.get(u'settings', {}), module).from_response(),
    }


def async_op_url(module, extra_data=None):
    if extra_data is None:
        extra_data = {}
    url = "https://www.googleapis.com/sql/v1beta4/projects/{project}/operations/{op_id}"
    combined = extra_data.copy()
    combined.update(module.params)
    return url.format(**combined)


def wait_for_operation(module, response):
    op_result = return_if_object(module, response, 'sql#operation')
    if op_result is None:
        return {}
    status = navigate_hash(op_result, ['status'])
    wait_done = wait_for_completion(status, op_result, module)
    return fetch_resource(module, navigate_hash(wait_done, ['targetLink']), 'sql#instance')


def wait_for_completion(status, op_result, module):
    op_id = navigate_hash(op_result, ['name'])
    op_uri = async_op_url(module, {'op_id': op_id})
    while status != 'DONE':
        raise_if_errors(op_result, ['error', 'errors'], module)
        time.sleep(1.0)
        op_result = fetch_resource(module, op_uri, 'sql#operation', False)
        status = navigate_hash(op_result, ['status'])
    return op_result


def raise_if_errors(response, err_path, module):
    errors = navigate_hash(response, err_path)
    if errors is not None:
        module.fail_json(msg=errors)


class InstanceFailoverreplica(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict({u'name': self.request.get('name')})

    def from_response(self):
        return remove_nones_from_dict({u'name': self.request.get(u'name')})


class InstanceIpaddressesArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({u'ipAddress': item.get('ip_address'), u'timeToRetire': item.get('time_to_retire'), u'type': item.get('type')})

    def _response_from_item(self, item):
        return remove_nones_from_dict({u'ipAddress': item.get(u'ipAddress'), u'timeToRetire': item.get(u'timeToRetire'), u'type': item.get(u'type')})


class InstanceReplicaconfiguration(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {
                u'failoverTarget': self.request.get('failover_target'),
                u'mysqlReplicaConfiguration': InstanceMysqlreplicaconfiguration(self.request.get('mysql_replica_configuration', {}), self.module).to_request(),
                u'replicaNames': self.request.get('replica_names'),
                u'serviceAccountEmailAddress': self.request.get('service_account_email_address'),
            }
        )

    def from_response(self):
        return remove_nones_from_dict(
            {
                u'failoverTarget': self.request.get(u'failoverTarget'),
                u'mysqlReplicaConfiguration': InstanceMysqlreplicaconfiguration(
                    self.request.get(u'mysqlReplicaConfiguration', {}), self.module
                ).from_response(),
                u'replicaNames': self.request.get(u'replicaNames'),
                u'serviceAccountEmailAddress': self.request.get(u'serviceAccountEmailAddress'),
            }
        )


class InstanceMysqlreplicaconfiguration(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {
                u'caCertificate': self.request.get('ca_certificate'),
                u'clientCertificate': self.request.get('client_certificate'),
                u'clientKey': self.request.get('client_key'),
                u'connectRetryInterval': self.request.get('connect_retry_interval'),
                u'dumpFilePath': self.request.get('dump_file_path'),
                u'masterHeartbeatPeriod': self.request.get('master_heartbeat_period'),
                u'password': self.request.get('password'),
                u'sslCipher': self.request.get('ssl_cipher'),
                u'username': self.request.get('username'),
                u'verifyServerCertificate': self.request.get('verify_server_certificate'),
            }
        )

    def from_response(self):
        return remove_nones_from_dict(
            {
                u'caCertificate': self.request.get(u'caCertificate'),
                u'clientCertificate': self.request.get(u'clientCertificate'),
                u'clientKey': self.request.get(u'clientKey'),
                u'connectRetryInterval': self.request.get(u'connectRetryInterval'),
                u'dumpFilePath': self.request.get(u'dumpFilePath'),
                u'masterHeartbeatPeriod': self.request.get(u'masterHeartbeatPeriod'),
                u'password': self.request.get(u'password'),
                u'sslCipher': self.request.get(u'sslCipher'),
                u'username': self.request.get(u'username'),
                u'verifyServerCertificate': self.request.get(u'verifyServerCertificate'),
            }
        )


class InstanceSettings(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {
                u'ipConfiguration': InstanceIpconfiguration(self.request.get('ip_configuration', {}), self.module).to_request(),
                u'tier': self.request.get('tier'),
                u'availabilityType': self.request.get('availability_type'),
                u'backupConfiguration': InstanceBackupconfiguration(self.request.get('backup_configuration', {}), self.module).to_request(),
            }
        )

    def from_response(self):
        return remove_nones_from_dict(
            {
                u'ipConfiguration': InstanceIpconfiguration(self.request.get(u'ipConfiguration', {}), self.module).from_response(),
                u'tier': self.request.get(u'tier'),
                u'availabilityType': self.request.get(u'availabilityType'),
                u'backupConfiguration': InstanceBackupconfiguration(self.request.get(u'backupConfiguration', {}), self.module).from_response(),
            }
        )


class InstanceIpconfiguration(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {
                u'ipv4Enabled': self.request.get('ipv4_enabled'),
                u'authorizedNetworks': InstanceAuthorizednetworksArray(self.request.get('authorized_networks', []), self.module).to_request(),
                u'requireSsl': self.request.get('require_ssl'),
            }
        )

    def from_response(self):
        return remove_nones_from_dict(
            {
                u'ipv4Enabled': self.request.get(u'ipv4Enabled'),
                u'authorizedNetworks': InstanceAuthorizednetworksArray(self.request.get(u'authorizedNetworks', []), self.module).from_response(),
                u'requireSsl': self.request.get(u'requireSsl'),
            }
        )


class InstanceAuthorizednetworksArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({u'expirationTime': item.get('expiration_time'), u'name': item.get('name'), u'value': item.get('value')})

    def _response_from_item(self, item):
        return remove_nones_from_dict({u'expirationTime': item.get(u'expirationTime'), u'name': item.get(u'name'), u'value': item.get(u'value')})


class InstanceBackupconfiguration(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {u'enabled': self.request.get('enabled'), u'binaryLogEnabled': self.request.get('binary_log_enabled'), u'startTime': self.request.get('start_time')}
        )

    def from_response(self):
        return remove_nones_from_dict(
            {u'enabled': self.request.get(u'enabled'), u'binaryLogEnabled': self.request.get(u'binaryLogEnabled'), u'startTime': self.request.get(u'startTime')}
        )


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2017,  Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class ModuleDocFragment(object):

    # Plugin option for AWS region
    DOCUMENTATION = r'''
options:
  region:
    description: The region for which to create the connection.
    type: str
    env:
      - name: EC2_REGION
      - name: AWS_REGION
'''



#!/usr/bin/python

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = """
module: na_ontap_net_subnet
short_description: NetApp ONTAP Create, delete, modify network subnets.
extends_documentation_fragment:
    - netapp.na_ontap
version_added: '2.8'
author:  Storage Engineering (@Albinpopote) <ansible@black-perl.fr>
description:
- Create, modify, destroy the network subnet
options:
  state:
    description:
    - Whether the specified network interface group should exist or not.
    choices: ['present', 'absent']
    default: present

  broadcast_domain:
    description:
    - Specify the required broadcast_domain name for the subnet.
    - A broadcast domain can not be modified after the subnet has been created
    required: true

  name:
    description:
    - Specify the subnet name.
    required: true

  from_name:
    description:
    - Name of the subnet to be renamed

  gateway:
    description:
    - Specify the gateway for the default route of the subnet.

  ipspace:
    description:
    - Specify the ipspace for the subnet.
    - The default value for this parameter is the default IPspace, named 'Default'.

  ip_ranges:
    description:
    - Specify the list of IP address ranges associated with the subnet.

  subnet:
    description:
    - Specify the subnet (ip and mask).
    required: true
"""

EXAMPLES = """
    - name: create subnet
      na_ontap_net_subnet:
        state: present
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"
        hostname: "{{ netapp_hostname }}"
        subnet: 10.10.10.0/24
        name: subnet-adm
        ip_ranges: [ '10.10.10.30-10.10.10.40', '10.10.10.51' ]
        gateway: 10.10.10.254
        ipspace: Default
        broadcast_domain: Default
    - name: delete subnet
      na_ontap_net_subnet:
        state: absent
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"
        hostname: "{{ netapp_hostname }}"
        name: subnet-adm
        ipspace: Default
    - name: rename subnet
      na_ontap_net_subnet:
        state: present
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"
        hostname: "{{ netapp_hostname }}"
        name: subnet-adm-new
        from_name: subnet-adm
        ipspace: Default
"""

RETURN = """

"""

import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
import ansible.module_utils.netapp as netapp_utils
from ansible.module_utils.netapp_module import NetAppModule

HAS_NETAPP_LIB = netapp_utils.has_netapp_lib()


class NetAppOntapSubnet(object):
    """
    Create, Modifies and Destroys a subnet
    """
    def __init__(self):
        """
        Initialize the ONTAP Subnet class
        """
        self.argument_spec = netapp_utils.na_ontap_host_argument_spec()
        self.argument_spec.update(dict(
            state=dict(required=False, choices=['present', 'absent'], default='present'),
            name=dict(required=True, type='str'),
            from_name=dict(required=False, type='str'),
            broadcast_domain=dict(required=False, type='str'),
            gateway=dict(required=False, type='str'),
            ip_ranges=dict(required=False, type=list),
            ipspace=dict(required=False, type='str'),
            subnet=dict(required=False, type='str')
        ))

        self.module = AnsibleModule(
            argument_spec=self.argument_spec,
            supports_check_mode=True
        )
        self.na_helper = NetAppModule()
        self.parameters = self.na_helper.set_parameters(self.module.params)

        if HAS_NETAPP_LIB is False:
            self.module.fail_json(msg="the python NetApp-Lib module is required")
        else:
            self.server = netapp_utils.setup_na_ontap_zapi(module=self.module)
        return

    def get_subnet(self, name=None):
        """
        Return details about the subnet
        :param:
            name : Name of the subnet
        :return: Details about the subnet. None if not found.
        :rtype: dict
        """
        if name is None:
            name = self.parameters.get('name')

        subnet_iter = netapp_utils.zapi.NaElement('net-subnet-get-iter')
        subnet_info = netapp_utils.zapi.NaElement('net-subnet-info')
        subnet_info.add_new_child('subnet-name', name)

        query = netapp_utils.zapi.NaElement('query')
        query.add_child_elem(subnet_info)

        subnet_iter.add_child_elem(query)

        result = self.server.invoke_successfully(subnet_iter, True)

        return_value = None
        # check if query returns the expected subnet
        if result.get_child_by_name('num-records') and \
                int(result.get_child_content('num-records')) == 1:

            subnet_attributes = result.get_child_by_name('attributes-list').get_child_by_name('net-subnet-info')
            broadcast_domain = subnet_attributes.get_child_content('broadcast-domain')
            gateway = subnet_attributes.get_child_content('gateway')
            ipspace = subnet_attributes.get_child_content('ipspace')
            subnet = subnet_attributes.get_child_content('subnet')
            name = subnet_attributes.get_child_content('subnet-name')

            ip_ranges = []
            range_obj = subnet_attributes.get_child_by_name('ip-ranges').get_children()
            for elem in range_obj:
                ip_ranges.append(elem.get_content())

            return_value = {
                'name': name,
                'broadcast_domain': broadcast_domain,
                'gateway': gateway,
                'ip_ranges': ip_ranges,
                'ipspace': ipspace,
                'subnet': subnet
            }

        return return_value

    def create_subnet(self):
        """
        Creates a new subnet
        """
        options = {'subnet-name': self.parameters.get('name'),
                   'broadcast-domain': self.parameters.get('broadcast_domain'),
                   'subnet': self.parameters.get('subnet')}
        subnet_create = netapp_utils.zapi.NaElement.create_node_with_children(
            'net-subnet-create', **options)

        if self.parameters.get('gateway'):
            subnet_create.add_new_child('gateway', self.parameters.get('gateway'))
        if self.parameters.get('ip_ranges'):
            subnet_ips = netapp_utils.zapi.NaElement('ip-ranges')
            subnet_create.add_child_elem(subnet_ips)
            for ip_range in self.parameters.get('ip_ranges'):
                subnet_ips.add_new_child('ip-range', ip_range)
        if self.parameters.get('ipspace'):
            subnet_create.add_new_child('ipspace', self.parameters.get('ipspace'))

        try:
            self.server.invoke_successfully(subnet_create, True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error creating subnet %s: %s' % (self.parameters.get('name'), to_native(error)),
                                  exception=traceback.format_exc())

    def delete_subnet(self):
        """
        Deletes a subnet
        """
        subnet_delete = netapp_utils.zapi.NaElement.create_node_with_children(
            'net-subnet-destroy', **{'subnet-name': self.parameters.get('name')})

        try:
            self.server.invoke_successfully(subnet_delete, True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error deleting subnet %s: %s' % (self.parameters.get('name'), to_native(error)),
                                  exception=traceback.format_exc())

    def modify_subnet(self):
        """
        Modifies a subnet
        """
        options = {'subnet-name': self.parameters.get('name')}

        subnet_modify = netapp_utils.zapi.NaElement.create_node_with_children(
            'net-subnet-modify', **options)

        if self.parameters.get('gateway'):
            subnet_modify.add_new_child('gateway', self.parameters.get('gateway'))
        if self.parameters.get('ip_ranges'):
            subnet_ips = netapp_utils.zapi.NaElement('ip-ranges')
            subnet_modify.add_child_elem(subnet_ips)
            for ip_range in self.parameters.get('ip_ranges'):
                subnet_ips.add_new_child('ip-range', ip_range)
        if self.parameters.get('ipspace'):
            subnet_modify.add_new_child('ipspace', self.parameters.get('ipspace'))
        if self.parameters.get('subnet'):
            subnet_modify.add_new_child('subnet', self.parameters.get('subnet'))

        try:
            self.server.invoke_successfully(subnet_modify, True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error modifying subnet %s: %s' % (self.parameters.get('name'), to_native(error)),
                                  exception=traceback.format_exc())

    def rename_subnet(self):
        """
        TODO
        """
        options = {'subnet-name': self.parameters.get('from_name'),
                   'new-name': self.parameters.get('name')}

        subnet_rename = netapp_utils.zapi.NaElement.create_node_with_children(
            'net-subnet-rename', **options)

        if self.parameters.get('ipspace'):
            subnet_rename.add_new_child('ipspace', self.parameters.get('ipspace'))

        try:
            self.server.invoke_successfully(subnet_rename, True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error renaming subnet %s: %s' % (self.parameters.get('name'), to_native(error)),
                                  exception=traceback.format_exc())

    def apply(self):
        '''Apply action to subnet'''
        current = self.get_subnet()
        cd_action, rename = None, None

        if self.parameters.get('from_name'):
            rename = self.na_helper.is_rename_action(self.get_subnet(self.parameters.get('from_name')), current)
            if rename is False:
                self.module.fail_json(msg="Error renaming: subnet %s does not exist" %
                                      self.parameters.get('from_name'))
        else:
            cd_action = self.na_helper.get_cd_action(current, self.parameters)

        modify = self.na_helper.get_modified_attributes(current, self.parameters)
        for attribute in modify:
            if attribute in ['broadcast_domain']:
                self.module.fail_json(msg='Error modifying subnet %s: cannot modify broadcast_domain parameter.' % self.parameters.get('name'))

        if self.na_helper.changed:
            if self.module.check_mode:
                pass
            else:
                if rename:
                    self.rename_subnet()
                # If rename is True, cd_action is NOne but modify could be true
                if cd_action == 'create':
                    for attribute in ['subnet', 'broadcast_domain']:
                        if not self.parameters.get(attribute):
                            self.module.fail_json(msg='Error - missing required arguments: %s.' % attribute)
                    self.create_subnet()
                elif cd_action == 'delete':
                    self.delete_subnet()
                elif modify:
                    self.modify_subnet()
        self.module.exit_json(changed=self.na_helper.changed)


def main():
    """
    Creates the NetApp ONTAP Net Route object and runs the correct play task
    """
    subnet_obj = NetAppOntapSubnet()
    subnet_obj.apply()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2017, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['stableinterface'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_device_ntp
short_description: Manage NTP servers on a BIG-IP
description:
  - Manage NTP servers on a BIG-IP.
version_added: 2.2
options:
  ntp_servers:
    description:
      - A list of NTP servers to set on the device. At least one of C(ntp_servers)
        or C(timezone) is required.
    type: list
  state:
    description:
      - The state of the NTP servers on the system. When C(present), guarantees
        that the NTP servers are set on the system. When C(absent), removes the
        specified NTP servers from the device configuration.
    type: str
    choices:
      - absent
      - present
    default: present
  timezone:
    description:
      - The timezone to set for NTP lookups. At least one of C(ntp_servers) or
        C(timezone) is required.
    type: str
extends_documentation_fragment: f5
author:
  - Tim Rupp (@caphrim007)
  - Wojciech Wypior (@wojtek0806)
'''

EXAMPLES = r'''
- name: Set NTP server
  bigip_device_ntp:
    ntp_servers:
      - 192.0.2.23
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost

- name: Set timezone
  bigip_device_ntp:
    timezone: America/Los_Angeles
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost
'''

RETURN = r'''
ntp_servers:
  description: The NTP servers that were set on the device
  returned: changed
  type: list
  sample: ["192.0.2.23", "192.0.2.42"]
timezone:
  description: The timezone that was set on the device
  returned: changed
  type: str
  sample: true
'''

from ansible.module_utils.basic import AnsibleModule

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import fq_name
    from library.module_utils.network.f5.common import f5_argument_spec
    from library.module_utils.network.f5.common import is_empty_list
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import fq_name
    from ansible.module_utils.network.f5.common import f5_argument_spec
    from ansible.module_utils.network.f5.common import is_empty_list


class Parameters(AnsibleF5Parameters):
    api_map = {
        'servers': 'ntp_servers',
    }

    api_attributes = [
        'servers', 'timezone',
    ]

    updatables = [
        'ntp_servers', 'timezone',
    ]

    returnables = [
        'ntp_servers', 'timezone',
    ]

    absentables = [
        'ntp_servers',
    ]


class ApiParameters(Parameters):
    pass


class ModuleParameters(Parameters):
    @property
    def ntp_servers(self):
        ntp_servers = self._values['ntp_servers']
        if ntp_servers is None:
            return None
        if is_empty_list(ntp_servers):
            return []
        return ntp_servers


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                change = getattr(self, returnable)
                if isinstance(change, dict):
                    result.update(change)
                else:
                    result[returnable] = change
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    pass


class ReportableChanges(Changes):
    pass


class Difference(object):
    def __init__(self, want, have=None):
        self.want = want
        self.have = have

    def compare(self, param):
        try:
            result = getattr(self, param)
            return result
        except AttributeError:
            return self.__default(param)

    def __default(self, param):
        attr1 = getattr(self.want, param)
        try:
            attr2 = getattr(self.have, param)
            if attr1 != attr2:
                return attr1
        except AttributeError:
            return attr1

    @property
    def ntp_servers(self):
        state = self.want.state
        if self.want.ntp_servers is None:
            return None
        if state == 'absent':
            if self.have.ntp_servers is None and self.want.ntp_servers:
                return None
            if set(self.want.ntp_servers) == set(self.have.ntp_servers):
                return []
            if set(self.want.ntp_servers) != set(self.have.ntp_servers):
                return list(set(self.want.ntp_servers).difference(self.have.ntp_servers))
        if not self.want.ntp_servers:
            if self.have.ntp_servers is None:
                return None
            if self.have.ntp_servers is not None:
                return self.want.ntp_servers
        if self.have.ntp_servers is None:
            return self.want.ntp_servers
        if set(self.want.ntp_servers) != set(self.have.ntp_servers):
            return self.want.ntp_servers


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.pop('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params)
        self.have = ApiParameters()
        self.changes = UsableChanges()

    def _announce_deprecations(self, result):
        warnings = result.pop('__warnings', [])
        for warning in warnings:
            self.module.deprecate(
                msg=warning['msg'],
                version=warning['version']
            )

    def _update_changed_options(self):
        diff = Difference(self.want, self.have)
        updatables = Parameters.updatables
        changed = dict()
        for k in updatables:
            change = diff.compare(k)
            if change is None:
                continue
            else:
                if isinstance(change, dict):
                    changed.update(change)
                else:
                    changed[k] = change
        if changed:
            self.changes = UsableChanges(params=changed)
            return True
        return False

    def _absent_changed_options(self):
        diff = Difference(self.want, self.have)
        absentables = Parameters.absentables
        changed = dict()
        for k in absentables:
            change = diff.compare(k)
            if change is None:
                continue
            else:
                if isinstance(change, dict):
                    changed.update(change)
                else:
                    changed[k] = change
        if changed:
            self.changes = UsableChanges(params=changed)
            return True
        return False

    def exec_module(self):
        changed = False
        result = dict()
        state = self.want.state

        if state == "present":
            changed = self.update()
        elif state == "absent":
            changed = self.absent()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=changed))
        self._announce_deprecations(result)
        return result

    def update(self):
        self.have = self.read_current_from_device()
        if not self.should_update():
            return False
        if self.module.check_mode:
            return True
        self.update_on_device()
        return True

    def should_update(self):
        result = self._update_changed_options()
        if result:
            return True
        return False

    def should_absent(self):
        result = self._absent_changed_options()
        if result:
            return True
        return False

    def absent(self):
        self.have = self.read_current_from_device()
        if not self.should_absent():
            return False
        if self.module.check_mode:
            return True
        self.absent_on_device()
        return True

    def update_on_device(self):
        params = self.changes.api_params()
        uri = "https://{0}:{1}/mgmt/tm/sys/ntp/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        resp = self.client.api.patch(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def read_current_from_device(self):
        uri = "https://{0}:{1}/mgmt/tm/sys/ntp/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return ApiParameters(params=response)

    def absent_on_device(self):
        params = self.changes.api_params()
        uri = "https://{0}:{1}/mgmt/tm/sys/ntp/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        resp = self.client.api.patch(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = True
        argument_spec = dict(
            ntp_servers=dict(
                type='list',
            ),
            timezone=dict(),
            state=dict(
                default='present',
                choices=['present', 'absent']
            ),
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)

        self.required_one_of = [
            ['ntp_servers', 'timezone']
        ]


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode,
        required_one_of=spec.required_one_of
    )

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# (c) 2018 Red Hat Inc.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


checkpoint_argument_spec = dict(auto_publish_session=dict(type='bool', default=True),
                                policy_package=dict(type='str', default='standard'),
                                auto_install_policy=dict(type='bool', default=True),
                                targets=dict(type='list')
                                )


def publish(connection, uid=None):
    payload = None

    if uid:
        payload = {'uid': uid}

    connection.send_request('/web_api/publish', payload)


def discard(connection, uid=None):
    payload = None

    if uid:
        payload = {'uid': uid}

    connection.send_request('/web_api/discard', payload)


def install_policy(connection, policy_package, targets):
    payload = {'policy-package': policy_package,
               'targets': targets}

    connection.send_request('/web_api/install-policy', payload)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Karsten Kaj Jakobsen <kj@patientsky.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = r'''
---
author:
  - "Karsten Kaj Jakobsen (@karstenjakobsen)"
description:
  - "This module can be used to create VM-Host rules in a given cluster."
extends_documentation_fragment: vmware.documentation
module: vmware_vm_host_drs_rule
notes:
  - "Tested on vSphere 6.5 and 6.7"
options:
  affinity_rule:
    default: true
    description:
      - "If set to C(True), the DRS rule will be an Affinity rule."
      - "If set to C(False), the DRS rule will be an Anti-Affinity rule."
      - "Effective only if C(state) is set to C(present)."
    type: bool
  datacenter:
    aliases:
      - datacenter_name
    description:
      - "Datacenter to search for given cluster. If not set, we use first cluster we encounter with C(cluster_name)."
    required: false
  cluster_name:
    description:
      - "Cluster to create VM-Host rule."
    required: true
  drs_rule_name:
    description:
      - "Name of rule to create or remove."
    required: true
  enabled:
    default: false
    description:
      - "If set to C(True), the DRS rule will be enabled."
      - "Effective only if C(state) is set to C(present)."
    type: bool
  host_group_name:
    description:
      - "Name of Host group to use with rule."
      - "Effective only if C(state) is set to C(present)."
    required: true
  mandatory:
    default: false
    description:
      - "If set to C(True), the DRS rule will be mandatory."
      - "Effective only if C(state) is set to C(present)."
    type: bool
  state:
    choices:
      - present
      - absent
    default: present
    description:
      - "If set to C(present) and the rule doesn't exists then the rule will be created."
      - "If set to C(absent) and the rule exists then the rule will be deleted."
    required: true
  vm_group_name:
    description:
      - "Name of VM group to use with rule."
      - "Effective only if C(state) is set to C(present)."
    required: true
requirements:
  - "python >= 2.6"
  - PyVmomi
short_description: "Creates vm/host group in a given cluster"
version_added: "2.8"

'''

EXAMPLES = r'''
---
- name: "Create mandatory DRS Affinity rule for VM/Host"
  vmware_vm_host_drs_rule:
    hostname: "{{ vcenter_hostname }}"
    password: "{{ vcenter_password }}"
    username: "{{ vcenter_username }}"
    validate_certs: False
    cluster_name: DC0_C0
    drs_rule_name: drs_rule_host_aff_0001
    host_group_name: DC0_C0_HOST_GR1
    vm_group_name: DC0_C0_VM_GR1
    mandatory: True
    enabled: True
    affinity_rule: True
'''

RETURN = r'''

'''

try:
    from pyVmomi import vim
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
from ansible.module_utils.vmware import (PyVmomi, vmware_argument_spec, wait_for_task, find_cluster_by_name,
                                         find_datacenter_by_name)


class VmwareVmHostRuleDrs(PyVmomi):
    """
    Class to manage VM HOST DRS Rules
    """

    def __init__(self, module):
        """
        Doctring: Init
        """

        super(VmwareVmHostRuleDrs, self).__init__(module)

        self.__datacenter_name = module.params.get('datacenter', None)
        self.__datacenter_obj = None
        self.__cluster_name = module.params['cluster_name']
        self.__cluster_obj = None
        self.__vm_group_name = module.params.get('vm_group_name', None)
        self.__host_group_name = module.params.get('host_group_name', None)
        self.__rule_name = module.params['drs_rule_name']
        self.__enabled = module.params['enabled']
        self.__mandatory = module.params['mandatory']
        self.__affinity_rule = module.params['affinity_rule']
        self.__state = module.params['state']
        self.__msg = 'Nothing to see here...'
        self.__result = dict()
        self.__changed = False

        if self.__datacenter_name is not None:

            self.__datacenter_obj = find_datacenter_by_name(self.content, self.__datacenter_name)

            if self.__datacenter_obj is None and module.check_mode is False:
                raise Exception("Datacenter '%s' not found" % self.__datacenter_name)

        self.__cluster_obj = find_cluster_by_name(content=self.content,
                                                  cluster_name=self.__cluster_name,
                                                  datacenter=self.__datacenter_obj)

        # Throw error if cluster does not exist
        if self.__cluster_obj is None and module.check_mode is False:
            raise Exception("Cluster '%s' not found" % self.__cluster_name)

        # Dont populate lists if we are deleting group
        if self.__state == 'present':
            # Get list of vm groups only if state is present
            self.__vm_group_obj = self.__get_group_by_name(group_name=self.__vm_group_name)
            self.__host_group_obj = self.__get_group_by_name(group_name=self.__host_group_name, host_group=True)

    def get_msg(self):
        """
        Returns message for Ansible result
        Args: none

        Returns: string
        """
        return self.__msg

    def get_result(self):
        """
        Returns result for Ansible
        Args: none

        Returns: dict
        """
        return self.__result

    def get_changed(self):
        """
        Returns if anything changed
        Args: none

        Returns: boolean
        """
        return self.__changed

    def __get_group_by_name(self, group_name, cluster_obj=None, host_group=False):
        """
        Return group
        Args:
            group_name: Group name
            cluster_obj: Cluster managed object

        Returns: cluster_obj.configurationEx.group

        """
        if cluster_obj is None:
            cluster_obj = self.__cluster_obj

        for group in cluster_obj.configurationEx.group:

            if not host_group and isinstance(group, vim.cluster.VmGroup):
                if group.name == group_name:
                    return group
            elif host_group and isinstance(group, vim.cluster.HostGroup):
                if group.name == group_name:
                    return group

        raise Exception("Failed to find the group %s in given cluster %s" % (group_name, cluster_obj.name))

    def __get_rule_key_by_name(self, cluster_obj=None, rule_name=None):
        """
        Function to get a specific VM-Host DRS rule key by name
        Args:
            rule_name: Name of rule
            cluster_obj: Cluster managed object

        Returns: Rule Object if found or None

        """

        if cluster_obj is None:
            cluster_obj = self.__cluster_obj

        if rule_name is None:
            rule_name = self.__rule_name

        if rule_name:
            rules_list = [rule for rule in cluster_obj.configuration.rule if rule.name == rule_name]
            if rules_list:
                return rules_list[0]

        # No rule found
        return None

    def __normalize_vm_host_rule_spec(self, rule_obj, cluster_obj=None):
        """
        Return human readable rule spec
        Args:
            rule_obj: Rule managed object
            cluster_obj: Cluster managed object

        Returns: Dictionary with VM-Host DRS Rule info

        """
        if cluster_obj is None:
            cluster_obj = self.__cluster_obj

        if not all([rule_obj, cluster_obj]):
            return {}

        return dict(rule_key=rule_obj.key,
                    rule_enabled=rule_obj.enabled,
                    rule_name=rule_obj.name,
                    rule_mandatory=rule_obj.mandatory,
                    rule_uuid=rule_obj.ruleUuid,
                    rule_vm_group_name=rule_obj.vmGroupName,
                    rule_affine_host_group_name=rule_obj.affineHostGroupName,
                    rule_anti_affine_host_group_name=rule_obj.antiAffineHostGroupName,
                    rule_vms=self.__get_all_from_group(group_name=rule_obj.vmGroupName,
                                                       cluster_obj=cluster_obj),
                    rule_affine_hosts=self.__get_all_from_group(group_name=rule_obj.affineHostGroupName,
                                                                cluster_obj=cluster_obj,
                                                                host_group=True),
                    rule_anti_affine_hosts=self.__get_all_from_group(group_name=rule_obj.antiAffineHostGroupName,
                                                                     cluster_obj=cluster_obj,
                                                                     host_group=True),
                    rule_type="vm_host_rule"
                    )

    def __get_all_from_group(self, group_name=None, cluster_obj=None, host_group=False):
        """
        Return all VM / Host names using given group name
        Args:
            group_name: Rule name
            cluster_obj: Cluster managed object
            host_group: True if we want only host name from group

        Returns: List of VM-Host names belonging to given group object

        """
        obj_name_list = []

        if not all([group_name, cluster_obj]):
            return obj_name_list

        for group in cluster_obj.configurationEx.group:
            if group.name == group_name:
                if not host_group and isinstance(group, vim.cluster.VmGroup):
                    obj_name_list = [vm.name for vm in group.vm]
                    break
                elif host_group and isinstance(group, vim.cluster.HostGroup):
                    obj_name_list = [host.name for host in group.host]
                    break

        return obj_name_list

    def __check_rule_has_changed(self, rule_obj, cluster_obj=None):
        """
        Function to check if the rule being edited has changed
        """

        if cluster_obj is None:
            cluster_obj = self.__cluster_obj

        existing_rule = self.__normalize_vm_host_rule_spec(rule_obj=rule_obj, cluster_obj=cluster_obj)

        # Check if anything has changed
        if ((existing_rule['rule_enabled'] == self.__enabled) and
            (existing_rule['rule_mandatory'] == self.__mandatory) and
            (existing_rule['rule_vm_group_name'] == self.__vm_group_name) and
            (existing_rule['rule_affine_host_group_name'] == self.__host_group_name or
             existing_rule['rule_anti_affine_host_group_name'] == self.__host_group_name)):

            return False
        else:
            return True

    def create(self):
        """
        Function to create a host VM-Host DRS rule if rule does not exist
        """
        rule_obj = self.__get_rule_key_by_name(rule_name=self.__rule_name)

        # Check if rule exists
        if rule_obj:

            operation = 'edit'
            rule_changed = self.__check_rule_has_changed(rule_obj)

        else:
            operation = 'add'

        # Check if anything has changed when editing
        if operation == 'add' or (operation == 'edit' and rule_changed is True):

            rule = vim.cluster.VmHostRuleInfo()

            # Check if already rule exists
            if rule_obj:
                # This need to be set in order to edit a existing rule
                rule.key = rule_obj.key

            rule.enabled = self.__enabled
            rule.mandatory = self.__mandatory
            rule.name = self.__rule_name

            if self.__affinity_rule:
                rule.affineHostGroupName = self.__host_group_name
            else:
                rule.antiAffineHostGroupName = self.__host_group_name

            rule.vmGroupName = self.__vm_group_name

            rule_spec = vim.cluster.RuleSpec(info=rule, operation=operation)
            config_spec = vim.cluster.ConfigSpecEx(rulesSpec=[rule_spec])

            if not self.module.check_mode:

                task = self.__cluster_obj.ReconfigureEx(config_spec, modify=True)
                wait_for_task(task)

            self.__changed = True

        rule_obj = self.__get_rule_key_by_name(rule_name=self.__rule_name)
        self.__result = self.__normalize_vm_host_rule_spec(rule_obj)

        if operation == 'edit':
            self.__msg = "Updated DRS rule `%s` successfully" % (self.__rule_name)
        else:
            self.__msg = "Created DRS rule `%s` successfully" % (self.__rule_name)

    # Delete
    def delete(self, rule_name=None):
        """
        Function to delete VM-Host DRS rule using name
        """
        changed = False

        if rule_name is None:
            rule_name = self.__rule_name

        rule_obj = self.__get_rule_key_by_name(rule_name=rule_name)

        if rule_obj is not None:

            rule_key = int(rule_obj.key)
            rule_spec = vim.cluster.RuleSpec(removeKey=rule_key, operation='remove')
            config_spec = vim.cluster.ConfigSpecEx(rulesSpec=[rule_spec])

            if not self.module.check_mode:

                task = self.__cluster_obj.ReconfigureEx(config_spec, modify=True)
                wait_for_task(task)

            self.__changed = True

        if self.__changed:
            self.__msg = "Deleted DRS rule `%s` successfully" % (self.__rule_name)
        else:
            self.__msg = "DRS Rule `%s` does not exists or already deleted" % (self.__rule_name)


def main():

    argument_spec = vmware_argument_spec()

    argument_spec.update(dict(
        state=dict(type='str', default='present', choices=['absent', 'present']),
        vm_group_name=dict(type='str', required=True),
        host_group_name=dict(type='str', required=True),
        cluster_name=dict(type='str', required=True),
        datacenter=dict(type='str', required=False, aliases=['datacenter_name']),
        drs_rule_name=dict(type='str', required=True),
        enabled=dict(type='bool', default=False),
        mandatory=dict(type='bool', default=False),
        affinity_rule=dict(type='bool', default=True))
    )

    required_if = [
        ['state', 'present', ['vm_group_name'], ['host_group_name']],
    ]

    module = AnsibleModule(argument_spec=argument_spec,
                           required_if=required_if,
                           supports_check_mode=True)

    try:
        # Create instance of VmwareDrsGroupManager
        vm_host_drs = VmwareVmHostRuleDrs(module=module)

        if module.params['state'] == 'present':
            vm_host_drs.create()
        elif module.params['state'] == 'absent':
            vm_host_drs.delete()

        # Set results
        results = dict(msg=vm_host_drs.get_msg(),
                       failed=False,
                       changed=vm_host_drs.get_changed(),
                       result=vm_host_drs.get_result())

    except Exception as error:
        results = dict(failed=True, msg="Error: `%s`" % error)

    if results['failed']:
        module.fail_json(**results)
    else:
        module.exit_json(**results)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# (c) 2018-2019, NetApp, Inc
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}


DOCUMENTATION = '''
module: na_ontap_quotas
short_description: NetApp ONTAP Quotas
extends_documentation_fragment:
    - netapp.na_ontap
version_added: '2.8'
author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
description:
- Set/Modify/Delete quota on ONTAP
options:
  state:
    description:
    - Whether the specified quota should exist or not.
    choices: ['present', 'absent']
    default: present
    type: str
  vserver:
    required: true
    description:
      - Name of the vserver to use.
    type: str
  volume:
    description:
    - The name of the volume that the quota resides on.
    required: true
    type: str
  quota_target:
    description:
    - The quota target of the type specified.
    required: true
    type: str
  qtree:
    description:
    - Name of the qtree for the quota.
    - For user or group rules, it can be the qtree name or "" if no qtree.
    - For tree type rules, this field must be "".
    default: ""
    type: str
  type:
    description:
    - The type of quota rule
    choices: ['user', 'group', 'tree']
    required: true
    type: str
  policy:
    description:
    - Name of the quota policy from which the quota rule should be obtained.
    type: str
  set_quota_status:
    description:
    - Whether the specified volume should have quota status on or off.
    type: bool
  file_limit:
    description:
    - The number of files that the target can have.
    default: '-'
    type: str
  disk_limit:
    description:
    - The amount of disk space that is reserved for the target.
    default: '-'
    type: str
  threshold:
    description:
    - The amount of disk space the target would have to exceed before a message is logged.
    default: '-'
    type: str
'''

EXAMPLES = """
    - name: Add/Set quota
      na_ontap_quotas:
        state: present
        vserver: ansible
        volume: ansible
        quota_target: /vol/ansible
        type: user
        policy: ansible
        file_limit: 2
        disk_limit: 3
        set_quota_status: True
        hostname: "{{ netapp_hostname }}"
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"
    - name: modify quota
      na_ontap_quotas:
        state: present
        vserver: ansible
        volume: ansible
        quota_target: /vol/ansible
        type: user
        policy: ansible
        file_limit: 2
        disk_limit: 3
        threshold: 3
        set_quota_status: False
        hostname: "{{ netapp_hostname }}"
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"
    - name: Delete quota
      na_ontap_quotas:
        state: absent
        vserver: ansible
        volume: ansible
        quota_target: /vol/ansible
        type: user
        policy: ansible
        hostname: "{{ netapp_hostname }}"
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"
"""

RETURN = """

"""


import traceback
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
import ansible.module_utils.netapp as netapp_utils
from ansible.module_utils.netapp_module import NetAppModule

HAS_NETAPP_LIB = netapp_utils.has_netapp_lib()


class NetAppONTAPQuotas(object):
    '''Class with quotas methods'''

    def __init__(self):

        self.argument_spec = netapp_utils.na_ontap_host_argument_spec()
        self.argument_spec.update(dict(
            state=dict(required=False, choices=['present', 'absent'], default='present'),
            vserver=dict(required=True, type='str'),
            volume=dict(required=True, type='str'),
            quota_target=dict(required=True, type='str'),
            qtree=dict(required=False, type='str', default=""),
            type=dict(required=True, type='str', choices=['user', 'group', 'tree']),
            policy=dict(required=False, type='str'),
            set_quota_status=dict(required=False, type='bool'),
            file_limit=dict(required=False, type='str', default='-'),
            disk_limit=dict(required=False, type='str', default='-'),
            threshold=dict(required=False, type='str', default='-')
        ))

        self.module = AnsibleModule(
            argument_spec=self.argument_spec,
            supports_check_mode=True
        )

        self.na_helper = NetAppModule()
        self.parameters = self.na_helper.set_parameters(self.module.params)

        if HAS_NETAPP_LIB is False:
            self.module.fail_json(
                msg="the python NetApp-Lib module is required")
        else:
            self.server = netapp_utils.setup_na_ontap_zapi(module=self.module, vserver=self.parameters['vserver'])

    def get_quota_status(self):
        """
        Return details about the quota status
        :param:
            name : volume name
        :return: status of the quota. None if not found.
        :rtype: dict
        """
        quota_status_get = netapp_utils.zapi.NaElement('quota-status')
        quota_status_get.translate_struct({
            'volume': self.parameters['volume']
        })
        try:
            result = self.server.invoke_successfully(quota_status_get, enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error fetching quotas status info: %s' % to_native(error),
                                  exception=traceback.format_exc())
        if result:
            return result['status']
        return None

    def get_quotas(self):
        """
        Get quota details
        :return: name of volume if quota exists, None otherwise
        """
        quota_get = netapp_utils.zapi.NaElement('quota-list-entries-iter')
        query = {
            'query': {
                'quota-entry': {
                    'volume': self.parameters['volume'],
                    'quota-target': self.parameters['quota_target'],
                    'quota-type': self.parameters['type']
                }
            }
        }
        quota_get.translate_struct(query)
        if self.parameters.get('policy'):
            quota_get['query']['quota-entry'].add_new_child('policy', self.parameters['policy'])
        try:
            result = self.server.invoke_successfully(quota_get, enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error fetching quotas info: %s' % to_native(error),
                                  exception=traceback.format_exc())
        if result.get_child_by_name('num-records') and int(result.get_child_content('num-records')) >= 1:
            return_values = {'volume': result['attributes-list']['quota-entry']['volume'],
                             'file_limit': result['attributes-list']['quota-entry']['file-limit'],
                             'disk_limit': result['attributes-list']['quota-entry']['disk-limit'],
                             'threshold': result['attributes-list']['quota-entry']['threshold']}
            return return_values
        return None

    def quota_entry_set(self):
        """
        Adds a quota entry
        """
        options = {'volume': self.parameters['volume'],
                   'quota-target': self.parameters['quota_target'],
                   'quota-type': self.parameters['type'],
                   'qtree': self.parameters['qtree'],
                   'file-limit': self.parameters['file_limit'],
                   'disk-limit': self.parameters['disk_limit'],
                   'threshold': self.parameters['threshold']}
        if self.parameters.get('policy'):
            options['policy'] = self.parameters['policy']
        set_entry = netapp_utils.zapi.NaElement.create_node_with_children(
            'quota-set-entry', **options)
        try:
            self.server.invoke_successfully(set_entry, enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error adding/modifying quota entry %s: %s'
                                  % (self.parameters['volume'], to_native(error)),
                                  exception=traceback.format_exc())

    def quota_entry_delete(self):
        """
        Deletes a quota entry
        """
        options = {'volume': self.parameters['volume'],
                   'quota-target': self.parameters['quota_target'],
                   'quota-type': self.parameters['type'],
                   'qtree': self.parameters['qtree']}
        set_entry = netapp_utils.zapi.NaElement.create_node_with_children(
            'quota-delete-entry', **options)
        if self.parameters.get('policy'):
            set_entry.add_new_child('policy', self.parameters['policy'])
        try:
            self.server.invoke_successfully(set_entry, enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error deleting quota entry %s: %s'
                                  % (self.parameters['volume'], to_native(error)),
                                  exception=traceback.format_exc())

    def quota_entry_modify(self, modify_attrs):
        """
        Modifies a quota entry
        """
        options = {'volume': self.parameters['volume'],
                   'quota-target': self.parameters['quota_target'],
                   'quota-type': self.parameters['type'],
                   'qtree': self.parameters['qtree']}
        options.update(modify_attrs)
        if self.parameters.get('policy'):
            options['policy'] = str(self.parameters['policy'])
        modify_entry = netapp_utils.zapi.NaElement.create_node_with_children(
            'quota-modify-entry', **options)
        try:
            self.server.invoke_successfully(modify_entry, enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error modifying quota entry %s: %s'
                                  % (self.parameters['volume'], to_native(error)),
                                  exception=traceback.format_exc())

    def on_or_off_quota(self, status):
        """
        on or off quota
        """
        quota = netapp_utils.zapi.NaElement.create_node_with_children(
            status, **{'volume': self.parameters['volume']})
        try:
            self.server.invoke_successfully(quota,
                                            enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error setting %s for %s: %s'
                                  % (status, self.parameters['volume'], to_native(error)),
                                  exception=traceback.format_exc())

    def apply(self):
        """
        Apply action to quotas
        """
        netapp_utils.ems_log_event("na_ontap_quotas", self.server)
        modify_quota_status = None
        modify_quota = None
        current = self.get_quotas()
        if 'set_quota_status' in self.parameters:
            quota_status = self.get_quota_status()
            if quota_status is not None:
                quota_status_action = self.na_helper.get_modified_attributes(
                    {'set_quota_status': True if quota_status == 'on' else False}, self.parameters)
                if quota_status_action:
                    modify_quota_status = 'quota-on' if quota_status_action['set_quota_status'] else 'quota-off'
        cd_action = self.na_helper.get_cd_action(current, self.parameters)
        if cd_action is None:
            modify_quota = self.na_helper.get_modified_attributes(current, self.parameters)
        if self.na_helper.changed:
            if self.module.check_mode:
                pass
            else:
                if cd_action == 'create':
                    self.quota_entry_set()
                elif cd_action == 'delete':
                    self.quota_entry_delete()
                elif modify_quota is not None:
                    for key in list(modify_quota):
                        modify_quota[key.replace("_", "-")] = modify_quota.pop(key)
                    self.quota_entry_modify(modify_quota)
                if modify_quota_status is not None:
                    self.on_or_off_quota(modify_quota_status)
        self.module.exit_json(changed=self.na_helper.changed)


def main():
    '''Execute action'''
    quota_obj = NetAppONTAPQuotas()
    quota_obj.apply()


if __name__ == '__main__':
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# (c) 2019 Red Hat Inc.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import os
import uuid

from ansible.module_utils.six import iteritems
from ansible.module_utils.six import iterkeys
from ansible.module_utils._text import to_text
from ansible.module_utils.basic import env_fallback

try:
    from skydive.graph import Node, Edge
    from skydive.rest.client import RESTClient
    from skydive.websocket.client import NodeAddedMsgType, NodeUpdatedMsgType, NodeDeletedMsgType
    from skydive.websocket.client import EdgeAddedMsgType, EdgeUpdatedMsgType, EdgeDeletedMsgType
    from skydive.websocket.client import WSClient, WSClientDefaultProtocol, WSMessage
    HAS_SKYDIVE_CLIENT = True
except ImportError:
    HAS_SKYDIVE_CLIENT = False

# defining skydive constants
SKYDIVE_GREMLIN_QUERY = 'G.V().Has'
SKYDIVE_GREMLIN_EDGE_QUERY = 'G.E().Has'

SKYDIVE_PROVIDER_SPEC = {
    'endpoint': dict(fallback=(env_fallback, ['SKYDIVE_ENDPOINT'])),
    'username': dict(fallback=(env_fallback, ['SKYDIVE_USERNAME'])),
    'password': dict(fallback=(env_fallback, ['SKYDIVE_PASSWORD']), no_log=True),
    'insecure': dict(type='bool', default=False, fallback=(env_fallback, ['SKYDIVE_INSECURE'])),
    'ssl': dict(type='bool', default=False, fallback=(env_fallback, ['SKYDIVE_SSL']))
}


class skydive_client_check(object):
    """ Base class for implementing Skydive Rest API """

    provider_spec = {'provider': dict(type='dict', options=SKYDIVE_PROVIDER_SPEC)}

    def __init__(self, **kwargs):
        ''' Base class for implementing Skydive Rest API '''
        if not HAS_SKYDIVE_CLIENT:
            raise Exception('skydive-client is required but does not appear '
                            'to be installed.  It can be installed using the '
                            'command `pip install skydive-client`')

        if not set(kwargs.keys()).issubset(SKYDIVE_PROVIDER_SPEC.keys()):
            raise Exception('invalid or unsupported keyword argument for skydive_restclient connection.')
        for key, value in iteritems(SKYDIVE_PROVIDER_SPEC):
            if key not in kwargs:
                # apply default values from SKYDIVE_PROVIDER_SPEC since we cannot just
                # assume the provider values are coming from AnsibleModule
                if 'default' in value:
                    kwargs[key] = value['default']
                # override any values with env variables unless they were
                # explicitly set
                env = ('SKYDIVE_%s' % key).upper()
                if env in os.environ:
                    kwargs[key] = os.environ.get(env)


class skydive_inject_protocol(object):
    """ Implements inject protocol for node and edge modules """

    def onOpen(self):
        module = self.factory.kwargs["module"]
        params = self.factory.kwargs["params"]
        result = self.factory.kwargs["result"]
        if "node1" and "node2" in self.factory.kwargs:
            node1 = self.factory.kwargs["node1"]
            node2 = self.factory.kwargs["node2"]

        if module.check_mode:
            self.stop()
            return
        try:
            host = params["host"]
            if params["metadata"]:
                metadata = module._check_type_dict(params["metadata"])
            else:
                metadata = {}
            if "node_type" in params:
                metadata["Name"] = params["name"]
                metadata["Type"] = params["node_type"]
                seed = params["seed"]
                if not seed:
                    seed = "%s:%s" % (params["name"], params["node_type"])
                if module.params['state'] == 'present' or module.params['state'] == 'update':
                    uid = str(uuid.uuid5(uuid.NAMESPACE_OID, seed))
                    node = Node(uid, host, metadata=metadata)
                    if module.params['state'] == 'present':
                        msg = WSMessage("Graph", NodeAddedMsgType, node)
                    else:
                        msg = WSMessage("Graph", NodeUpdatedMsgType, node)
                else:
                    uid = params['id']
                    node = Node(uid, host, metadata=metadata)
                    msg = WSMessage("Graph", NodeDeletedMsgType, node)
            elif "relation_type" in params:
                metadata["RelationType"] = params["relation_type"]
                if module.params['state'] == 'present' or module.params['state'] == 'update':
                    uid = str(uuid.uuid5(uuid.NAMESPACE_OID, "%s:%s:%s" %
                                         (node1, node2, params["relation_type"])))
                    edge = Edge(uid, host, node1, node2, metadata=metadata)
                    if module.params['state'] == 'present':
                        msg = WSMessage("Graph", EdgeAddedMsgType, edge)
                    else:
                        msg = WSMessage("Graph", EdgeUpdatedMsgType, edge)
                else:
                    uid = module.params['id']
                    edge = Edge(uid, host, node1, node2, metadata=metadata)
                    msg = WSMessage("Graph", EdgeDeletedMsgType, edge)

            self.sendWSMessage(msg)
            if uid:
                result["UUID"] = uid
            result["changed"] = True
        except Exception as e:
            module.fail_json(
                msg='Error during topology update %s' % e, **result)
        finally:
            self.stop()


class skydive_wsclient(skydive_client_check):
    """ Base class for implementing Skydive Websocket API """

    def __init__(self, module, **kwargs):
        super(skydive_wsclient, self).__init__(**kwargs)

        class skydive_full_inject_protocol(skydive_inject_protocol, WSClientDefaultProtocol):
            pass
        kwargs['scheme'] = "ws"
        if 'ssl' in kwargs:
            if kwargs['ssl']:
                kwargs['scheme'] = "wss"
        if 'insecure' not in kwargs:
            kwargs['insecure'] = False
        scheme = kwargs['scheme']
        self.result = dict(changed=False)
        if "node_type" in module.params:
            self.wsclient_object = WSClient("ansible-" + str(os.getpid()) + "-" + module.params['host'],
                                            "%s://%s/ws/publisher" % (scheme, kwargs["endpoint"]),
                                            protocol=type('skydive_full_inject_protocol', (skydive_inject_protocol,
                                                                                           WSClientDefaultProtocol), dict()),
                                            persistent=True,
                                            insecure=kwargs["insecure"],
                                            username=kwargs["username"],
                                            password=kwargs["password"],
                                            module=module,
                                            params=module.params,
                                            result=self.result)
        elif "relation_type" in module.params:
            self.parent_node = self.get_node_id(module.params['parent_node'])
            self.child_node = self.get_node_id(module.params['child_node'])

            self.wsclient_object = WSClient("ansible-" + str(os.getpid()) + "-" + module.params['host'],
                                            "%s://%s/ws/publisher" % (scheme, kwargs["endpoint"]),
                                            protocol=type('skydive_full_inject_protocol', (skydive_inject_protocol,
                                                                                           WSClientDefaultProtocol), dict()),
                                            persistent=True,
                                            insecure=kwargs["insecure"],
                                            username=kwargs["username"],
                                            password=kwargs["password"],
                                            module=module,
                                            params=module.params,
                                            node1=self.parent_node,
                                            node2=self.child_node,
                                            result=self.result)

    def get_node_id(self, node_selector):
        """ Checks if Gremlin expresssion is passed as input to get the nodes UUID """
        if node_selector.startswith("G.") or node_selector.startswith("g."):
            nodes = self.restclient_object.lookup_nodes(node_selector)
            if len(nodes) == 0:
                raise self.module.fail_json(msg=to_text("Node not found: {0}".format(node_selector)))
            elif len(nodes) > 1:
                raise self.module.fail_json(
                    msg=to_text("Node selection should return only one node: {0}".format(node_selector)))
            return str(nodes[0].id)
        return node_selector


class skydive_restclient(skydive_client_check):
    """ Base class for implementing Skydive Rest API """

    def __init__(self, **kwargs):
        super(skydive_restclient, self).__init__(**kwargs)
        kwargs['scheme'] = "http"
        if 'ssl' in kwargs:
            if kwargs['ssl']:
                kwargs['scheme'] = "https"
        if 'insecure' not in kwargs:
            kwargs['insecure'] = False
        self.restclient_object = RESTClient(kwargs['endpoint'],
                                            scheme=kwargs['scheme'],
                                            insecure=kwargs['insecure'],
                                            username=kwargs['username'],
                                            password=kwargs['password'])


class skydive_lookup(skydive_restclient):
    """ Implements Skydive Lookup queries """

    provider_spec = {'provider': dict(type='dict', options=SKYDIVE_PROVIDER_SPEC)}

    def __init__(self, provider):
        super(skydive_lookup, self).__init__(**provider)
        self.query_str = ""

    def lookup_query(self, filter_data):
        query_key = filter_data.keys()[0]
        self.query_str = filter_data[query_key]
        nodes = self.restclient_object.lookup_nodes(self.query_str)
        result = []
        for each in nodes:
            result.append(each.__dict__)
        if len(result) == 0:
            raise Exception("Cannot find any entry for the input Gremlin query!")
        return result


class skydive_flow_capture(skydive_restclient):
    """ Implements Skydive Flow capture modules """

    def __init__(self, module):
        self.module = module
        provider = module.params['provider']

        super(skydive_flow_capture, self).__init__(**provider)

    def run(self, ib_spec):
        state = self.module.params['state']
        if state not in ('present', 'absent'):
            self.module.fail_json(msg='state must be one of `present`, `absent`, got `%s`' % state)

        result = {'changed': False}
        obj_filter = dict([(k, self.module.params[k]) for k, v in iteritems(ib_spec) if v.get('ib_req')])

        proposed_object = {}
        for key in iterkeys(ib_spec):
            if self.module.params[key] is not None:
                proposed_object[key] = self.module.params[key]

        if obj_filter['query']:
            cature_query = obj_filter['query']
        elif obj_filter['interface_name'] and obj_filter['type']:
            cature_query = SKYDIVE_GREMLIN_QUERY + "('Name', '{0}', 'Type', '{1}')".format(obj_filter['interface_name'],
                                                                                           obj_filter['type'])
        else:
            raise self.module.fail_json(msg="Interface name and Type is required if gremlin query is not defined!")

        # to check current object ref for idempotency
        captured_list_objs = self.restclient_object.capture_list()
        current_ref_uuid = None
        for each_capture in captured_list_objs:
            if cature_query == each_capture.__dict__['query']:
                current_ref_uuid = each_capture.__dict__['uuid']
                break
        if state == 'present':
            if not current_ref_uuid:
                try:
                    self.restclient_object.capture_create(cature_query, obj_filter['capture_name'],
                                                          obj_filter['description'], obj_filter['extra_tcp_metric'],
                                                          obj_filter['ip_defrag'], obj_filter['reassemble_tcp'],
                                                          obj_filter['layer_key_mode'])
                except Exception as e:
                    self.module.fail_json(msg=to_text(e))
                result['changed'] = True
        if state == 'absent':
            if current_ref_uuid:
                try:
                    self.restclient_object.capture_delete(current_ref_uuid)
                except Exception as e:
                    self.module.fail_json(msg=to_text(e))
                result['changed'] = True

        return result


class skydive_node(skydive_wsclient, skydive_restclient):
    """ Implements Skydive Node modules """

    def __init__(self, module):
        self.module = module
        provider = module.params['provider']
        super(skydive_node, self).__init__(self.module, **provider)

    def run(self):
        try:
            lookup_query = SKYDIVE_GREMLIN_QUERY + "('Name', '{0}', 'Type', '{1}')".format(self.module.params['name'],
                                                                                           self.module.params['node_type'])
            node_exists = self.restclient_object.lookup_nodes(lookup_query)

            if not node_exists and self.module.params['state'] == 'present':
                self.wsclient_object.connect()
                self.wsclient_object.start()
            elif len(node_exists) > 0 and self.module.params['state'] == 'update':
                self.wsclient_object.connect()
                self.wsclient_object.start()
            elif len(node_exists) > 0 and self.module.params['state'] == 'absent':
                self.module.params['id'] = node_exists[0].__dict__['id']
                self.wsclient_object.connect()
                self.wsclient_object.start()
        except Exception as e:
            self.module.fail_json(msg=to_text(e))
        return self.result


class skydive_edge(skydive_wsclient, skydive_restclient):
    """ Implements Skydive Edge modules """

    def __init__(self, module):
        self.module = module
        provider = module.params['provider']

        super(skydive_edge, self).__init__(self.module, **provider)

    def run(self):
        try:
            edge_exists = False
            edge_query = SKYDIVE_GREMLIN_EDGE_QUERY + "('Parent', '{0}', 'Child', '{1}')".format(self.parent_node,
                                                                                                 self.child_node)
            query_result = self.restclient_object.lookup_edges(edge_query)
            if query_result:
                query_result = query_result[0].__dict__
                edge_exists = True

            if not edge_exists and self.module.params['state'] == 'present':
                self.wsclient_object.connect()
                self.wsclient_object.start()
            elif edge_exists and self.module.params['state'] == 'update':
                self.wsclient_object.connect()
                self.wsclient_object.start()
            elif edge_exists and self.module.params['state'] == 'absent':
                self.module.params['id'] = query_result['id']
                self.wsclient_object.connect()
                self.wsclient_object.start()
        except Exception as e:
            self.module.fail_json(msg=to_text(e))
        return self.result



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_firewall_multicast_policy6
short_description: Configure IPv6 multicast NAT policies in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure firewall feature and multicast_policy6 category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    firewall_multicast_policy6:
        description:
            - Configure IPv6 multicast NAT policies.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            action:
                description:
                    - Accept or deny traffic matching the policy.
                choices:
                    - accept
                    - deny
            dstaddr:
                description:
                    - IPv6 destination address name.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.multicast-address6.name.
                        required: true
            dstintf:
                description:
                    - IPv6 destination interface name. Source system.interface.name system.zone.name.
            end-port:
                description:
                    - Integer value for ending TCP/UDP/SCTP destination port in range (1 - 65535, default = 65535).
            id:
                description:
                    - Policy ID.
                required: true
            logtraffic:
                description:
                    - Enable/disable logging traffic accepted by this policy.
                choices:
                    - enable
                    - disable
            protocol:
                description:
                    - Integer value for the protocol type as defined by IANA (0 - 255, default = 0).
            srcaddr:
                description:
                    - IPv6 source address name.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address6.name firewall.addrgrp6.name.
                        required: true
            srcintf:
                description:
                    - IPv6 source interface name. Source system.interface.name system.zone.name.
            start-port:
                description:
                    - Integer value for starting TCP/UDP/SCTP destination port in range (1 - 65535, default = 1).
            status:
                description:
                    - Enable/disable this policy.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPv6 multicast NAT policies.
    fortios_firewall_multicast_policy6:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      firewall_multicast_policy6:
        state: "present"
        action: "accept"
        dstaddr:
         -
            name: "default_name_5 (source firewall.multicast-address6.name)"
        dstintf: "<your_own_value> (source system.interface.name system.zone.name)"
        end-port: "7"
        id:  "8"
        logtraffic: "enable"
        protocol: "10"
        srcaddr:
         -
            name: "default_name_12 (source firewall.address6.name firewall.addrgrp6.name)"
        srcintf: "<your_own_value> (source system.interface.name system.zone.name)"
        start-port: "14"
        status: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_firewall_multicast_policy6_data(json):
    option_list = ['action', 'dstaddr', 'dstintf',
                   'end-port', 'id', 'logtraffic',
                   'protocol', 'srcaddr', 'srcintf',
                   'start-port', 'status']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def firewall_multicast_policy6(data, fos):
    vdom = data['vdom']
    firewall_multicast_policy6_data = data['firewall_multicast_policy6']
    filtered_data = filter_firewall_multicast_policy6_data(firewall_multicast_policy6_data)
    if firewall_multicast_policy6_data['state'] == "present":
        return fos.set('firewall',
                       'multicast-policy6',
                       data=filtered_data,
                       vdom=vdom)

    elif firewall_multicast_policy6_data['state'] == "absent":
        return fos.delete('firewall',
                          'multicast-policy6',
                          mkey=filtered_data['id'],
                          vdom=vdom)


def fortios_firewall(data, fos):
    login(data)

    methodlist = ['firewall_multicast_policy6']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "firewall_multicast_policy6": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "action": {"required": False, "type": "str",
                           "choices": ["accept", "deny"]},
                "dstaddr": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "dstintf": {"required": False, "type": "str"},
                "end-port": {"required": False, "type": "int"},
                "id": {"required": True, "type": "int"},
                "logtraffic": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "protocol": {"required": False, "type": "int"},
                "srcaddr": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "srcintf": {"required": False, "type": "str"},
                "start-port": {"required": False, "type": "int"},
                "status": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_firewall(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2018, CrySyS Lab <www.crysys.hu>
# Copyright: (c) 2018, Peter Gyorgy <gyorgy.peter@edu.bme.hu>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = '''
---
module: vmware_vspan_session
short_description: Create or remove a Port Mirroring session.
description:
   - This module can be used to create, delete or edit different kind of port mirroring sessions.
version_added: '2.8'
author:
- Peter Gyorgy (@gyorgypeter) <gyorgy.peter1996@gmail.com>
notes:
    - Tested on vSphere 6.7
requirements:
    - "python > =  2.6"
    - PyVmomi
options:
    switch:
        description:
            - The name of the distributed vSwitch on which to add or remove the mirroring session.
        required: True
        aliases: [ 'switch_name' ]
    name:
        description:
            - Name of the session.
        required: True
    state:
        choices:
            - 'present'
            - 'absent'
        description:
            - Create or remove the session.
        required: True
    session_type:
        default: 'dvPortMirror'
        choices:
            - 'encapsulatedRemoteMirrorSource'
            - 'remoteMirrorDest'
            - 'remoteMirrorSource'
            - 'dvPortMirror'
        description:
            - Select the mirroring type.
            - '- C(encapsulatedRemoteMirrorSource) (str): In encapsulatedRemoteMirrorSource session, Distributed Ports
            can be used as source entities, and Ip address can be used as destination entities.'
            - '- C(remoteMirrorDest) (str): In remoteMirrorDest session, vlan Ids can be used as source entities, and
            Distributed Ports can be used as destination entities.'
            - '- C(remoteMirrorSource) (str): In remoteMirrorSource session, Distributed Ports can be used as source
            entities, and uplink ports name can be used as destination entities.'
            - '- C(dvPortMirror) (str): In dvPortMirror session, Distributed Ports can be used as both source and
            destination entities.'
        required: False
    enabled:
        type: bool
        default: True
        description:
            - Whether the session is enabled.
    description:
        description:
            - The description for the session.
        required: False
    source_port_transmitted:
        description:
            - Source port for which transmitted packets are mirrored.
        required: False
    source_port_received:
        description:
            - Source port for which received packets are mirrored.
        required: False
    destination_port:
        description:
            - Destination port that received the mirrored packets. Also any port designated in the value of this
             property can not match the source port in any of the Distributed Port Mirroring session.
        required: False
    encapsulation_vlan_id:
        description:
            - VLAN ID used to encapsulate the mirrored traffic.
        required: False
    strip_original_vlan:
        description:
            - Whether to strip the original VLAN tag. if false, the original VLAN tag will be preserved on the mirrored
             traffic. If encapsulationVlanId has been set and this property is false, the frames will be double tagged
             with the original VLAN ID as the inner tag.
        type: bool
        required: False
    mirrored_packet_length:
        description:
            - An integer that describes how much of each frame to mirror. If unset, all of the frame would be mirrored.
             Setting this property to a smaller value is useful when the consumer will look only at the headers.
             The value cannot be less than 60.
        required: False
    normal_traffic_allowed:
        description:
            - Whether or not destination ports can send and receive "normal" traffic. Setting this to false will make
             mirror ports be used solely for mirroring and not double as normal access ports.
        type: bool
        required: False
    sampling_rate:
        description:
            - Sampling rate of the session. If its value is n, one of every n packets is mirrored.
             Valid values are between 1 to 65535, and default value is 1.
        type: int
        required: False
    source_vm_transmitted:
        description:
            - With this parameter it is possible, to add a NIC of a VM to a port mirroring session.
            - 'Valid attributes are:'
            - '- C(name) (str): Name of the VM'
            - '- C(nic_label) (bool): Label of the Network Interface Card to use.'
    source_vm_received:
        description:
            - With this parameter it is possible, to add a NIC of a VM to a port mirroring session.
            - 'Valid attributes are:'
            - '- C(name) (str): Name of the VM'
            - '- C(nic_label) (bool): Label of the Network Interface Card to use.'
    destination_vm:
        description:
            - With this parameter it is possible, to add a NIC of a VM to a port mirroring session.
            - 'Valid attributes are:'
            - '- C(name) (str): Name of the VM'
            - '- C(nic_label) (bool): Label of the Network Interface Card to use.'
        required: False
extends_documentation_fragment: vmware.documentation
'''

EXAMPLES = '''
- name: Create distributed mirroring session.
  vmware_vspan_session:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    switch_name: dvSwitch
    state: present
    name: Basic Session
    enabled: True
    description: "Example description"
    source_port_transmitted: 817
    source_port_received: 817
    destination_port: 815
  delegate_to: localhost

- name: Create remote destination mirroring session.
  vmware_vspan_session:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    switch_name: dvSwitch
    state: present
    name: Remote Session
    enabled: True
    description: "Example description"
    source_port_received: 105
    destination_port: 815
    session_type: "remoteMirrorDest"
  delegate_to: localhost

- name: Create remote destination mirroring session.
  vmware_vspan_session:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    switch_name: dvSwitch
    state: absent
    name: Remote Session
  delegate_to: localhost
'''

RETURN = """#
"""

try:
    from pyVmomi import vim, vmodl
except ImportError as e:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.vmware import (vmware_argument_spec, PyVmomi, find_dvs_by_name,
                                         find_vm_by_name, wait_for_task)


class VMwareVspanSession(PyVmomi):
    def __init__(self, module):
        super(VMwareVspanSession, self).__init__(module)
        self.switch = module.params['switch']
        self.name = module.params['name']
        self.session_type = module.params['session_type']
        self.enabled = module.params['enabled']
        self.state = module.params['state']
        self.description = module.params['description']
        self.source_port_transmitted = module.params['source_port_transmitted']
        self.source_port_received = module.params['source_port_received']
        self.destination_port = module.params['destination_port']
        self.encapsulation_vlan_id = module.params['encapsulation_vlan_id']
        self.strip_original_vlan = module.params['strip_original_vlan']
        self.mirrored_packet_length = module.params['mirrored_packet_length']
        self.normal_traffic_allowed = module.params['normal_traffic_allowed']
        self.sampling_rate = module.params['sampling_rate']
        self.dv_switch = find_dvs_by_name(self.content, self.switch)
        if self.dv_switch is None:
            self.module.fail_json(msg="There is no dvSwitch with the name: {0:s}.".format(self.switch))
        self.operation = None
        self.modified_ports = dict()
        self.deleted_session = None
        if module.params['source_vm_transmitted'] is not None:
            if (module.params['source_vm_transmitted']['name'] is None or
                    module.params['source_vm_transmitted']['nic_label'] is None):
                self.module.fail_json(msg="Please provide both VM name and NIC Label")
            self.source_vm_transmitted_name = module.params['source_vm_transmitted']['name']
            self.source_vm_transmitted_nic_label = module.params['source_vm_transmitted']['nic_label']
        if module.params['source_vm_received'] is not None:
            if (module.params['source_vm_received']['name'] is None or
                    module.params['source_vm_received']['nic_label'] is None):
                self.module.fail_json(msg="Please provide both VM name and NIC Label")
            self.source_vm_received_name = module.params['source_vm_received']['name']
            self.source_vm_received_nic_label = module.params['source_vm_received']['nic_label']
        if module.params['destination_vm'] is not None:
            if (module.params['destination_vm']['name'] is None or
                    module.params['destination_vm']['nic_label'] is None):
                self.module.fail_json(msg="Please provide both VM name and NIC Label")
            self.destination_vm_name = module.params['destination_vm']['name']
            self.destination_vm_nic_label = module.params['destination_vm']['nic_label']

    def set_operation(self):
        """Sets the operation according to state"""
        if self.state == 'absent':
            self.operation = 'remove'
        elif self.state == 'present' and self.find_session_by_name() is None:
            self.operation = 'add'
        else:
            self.operation = 'edit'

    def find_session_by_name(self):
        """Finds a session by name
        Returns
        -------
        vim.dvs.VmwareDistributedVirtualSwitch.VspanSession
            The session if there was a session by the given name, else returns None
        """
        for vspan_session in self.dv_switch.config.vspanSession:
            if vspan_session.name == self.name:
                return vspan_session
        return None

    def get_vm_port(self, vm_name, nic_label):
        """Finds the port of the VM
        Returns
        -------
        str
            the port number as a string, or None if the NIC couldnt be found
        """
        vm = find_vm_by_name(self.content, vm_name)
        if vm is None:
            self.module.fail_json(msg="There is no VM with the name: {0:s}.".format(vm_name))
        for hardware in vm.config.hardware.device:
            if isinstance(hardware, vim.vm.device.VirtualVmxnet3):
                if hardware.deviceInfo.label == nic_label:
                    return hardware.backing.port.portKey
        return None

    def set_port_for_vm(self):
        """Sets the ports, to the VM's specified port."""
        if hasattr(self, 'source_vm_transmitted_name') and hasattr(self, 'source_vm_transmitted_nic_label'):
            port = self.get_vm_port(self.source_vm_transmitted_name, self.source_vm_transmitted_nic_label)
            if port is not None:
                self.source_port_transmitted = port
            else:
                self.module.fail_json(
                    msg="No port could be found for VM: {0:s} NIC: {1:s}".format(self.source_vm_transmitted_name,
                                                                                 self.source_vm_transmitted_nic_label))
        if hasattr(self, 'source_vm_received_name') and hasattr(self, 'source_vm_received_nic_label'):
            port = self.get_vm_port(self.source_vm_received_name, self.source_vm_received_nic_label)
            if port is not None:
                self.source_port_received = port
            else:
                self.module.fail_json(
                    msg="No port could be found for VM: {0:s} NIC: {1:s}".format(self.source_vm_received_name,
                                                                                 self.source_vm_received_nic_label))
        if hasattr(self, 'destination_vm_name') and hasattr(self, 'destination_vm_nic_label'):
            port = self.get_vm_port(self.destination_vm_name, self.destination_vm_nic_label)
            if port is not None:
                self.destination_port = port
            else:
                self.module.fail_json(
                    msg="No port could be found for VM: {0:s} NIC: {1:s}".format(self.destination_vm_name,
                                                                                 self.destination_vm_nic_label))

    def process_operation(self):
        """Calls the create or delete function based on the operation"""
        self.set_operation()
        if self.operation == 'remove':
            results = self.remove_vspan_session()
            self.module.exit_json(**results)
        if self.operation == 'add':
            self.set_port_for_vm()
            results = self.add_vspan_session()
            self.module.exit_json(**results)
        if self.operation == 'edit':
            self.remove_vspan_session()
            self.set_port_for_vm()
            results = self.add_vspan_session()
            self.module.exit_json(**results)

    def set_port_security_promiscuous(self, ports, state):
        """Set the given port to the given promiscuous state.
        Parameters
        ----------
        port : str[]
            PortKey
        state: bool
            State of the promiscuous mode, if true its allowed, else not.
        """
        # Creating the new port policy
        port_spec = []
        vim_bool = vim.BoolPolicy(value=state)
        port_policy = vim.dvs.VmwareDistributedVirtualSwitch.SecurityPolicy(allowPromiscuous=vim_bool)
        port_settings = vim.dvs.VmwareDistributedVirtualSwitch.VmwarePortConfigPolicy(securityPolicy=port_policy)
        for port in ports:
            temp_port_spec = vim.dvs.DistributedVirtualPort.ConfigSpec(
                operation="edit",
                key=port,
                setting=port_settings
            )
            port_spec.append(temp_port_spec)

        task = self.dv_switch.ReconfigureDVPort_Task(port_spec)
        try:
            wait_for_task(task)
        except Exception:
            self.restore_original_state()
            self.module.fail_json(msg=task.info.error.msg)

    def turn_off_promiscuous(self):
        """Disable all promiscuous mode ports, and give them back in a list.
        Returns
        -------
        list
            Contains every port, where promiscuous mode has been turned off
        """
        # Ports that are in mirror sessions
        ports = []
        ports_of_selected_session = []
        for vspan_session in self.dv_switch.config.vspanSession:
            if vspan_session.sourcePortReceived is not None:
                session_ports = vspan_session.sourcePortReceived.portKey
                for port in session_ports:
                    if vspan_session.name == self.name:
                        ports_of_selected_session.append(port)
                    elif not(port in ports):
                        ports.append(port)
            if vspan_session.sourcePortTransmitted is not None:
                session_ports = vspan_session.sourcePortTransmitted.portKey
                for port in session_ports:
                    if vspan_session.name == self.name:
                        ports_of_selected_session.append(port)
                    elif not(port in ports):
                        ports.append(port)
            if vspan_session.destinationPort is not None:
                session_ports = vspan_session.destinationPort.portKey
                for port in session_ports:
                    if vspan_session.name == self.name:
                        ports_of_selected_session.append(port)
                    elif not(port in ports):
                        ports.append(port)
        promiscuous_ports = []
        if ports:
            dv_ports = self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=ports))
            # If a port is promiscuous set disable it, and add it to the array to enable it after the changes are made.
            for dv_port in dv_ports:
                if dv_port.config.setting.securityPolicy.allowPromiscuous.value:
                    self.set_port_security_promiscuous([dv_port.key], False)
                    self.modified_ports.update({dv_port.key: True})
                    promiscuous_ports.append(dv_port.key)
        if ports_of_selected_session:
            current_dv_ports = self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=ports_of_selected_session))
            for dv_port in current_dv_ports:
                if dv_port.config.setting.securityPolicy.allowPromiscuous.value:
                    self.set_port_security_promiscuous([dv_port.key], False)
                    self.modified_ports.update({dv_port.key: True})
        # Return the promiscuous ports array, to set them back after the config is finished.
        return promiscuous_ports

    def delete_mirroring_session(self, key):
        """Deletes the mirroring session.
        Parameters
        ----------
        key : str
            Key of the Session
        """
        session = vim.dvs.VmwareDistributedVirtualSwitch.VspanSession(
            key=key
        )
        config_version = self.dv_switch.config.configVersion
        s_spec = vim.dvs.VmwareDistributedVirtualSwitch.VspanConfigSpec(vspanSession=session, operation="remove")
        c_spec = vim.dvs.VmwareDistributedVirtualSwitch.ConfigSpec(vspanConfigSpec=[s_spec], configVersion=config_version)
        task = self.dv_switch.ReconfigureDvs_Task(c_spec)
        try:
            wait_for_task(task)
        except Exception:
            self.restore_original_state()
            self.module.fail_json(msg=task.info.error.msg)

    def restore_original_state(self):
        """In case of failure restore, the changes we made."""
        for port, state in self.modified_ports.items():
            self.set_port_security_promiscuous([port], state)
        if self.deleted_session is not None:
            session = self.deleted_session
            config_version = self.dv_switch.config.configVersion
            s_spec = vim.dvs.VmwareDistributedVirtualSwitch.VspanConfigSpec(vspanSession=session, operation="add")
            c_spec = vim.dvs.VmwareDistributedVirtualSwitch.ConfigSpec(vspanConfigSpec=[s_spec], configVersion=config_version)
            # Revert the delete
            task = self.dv_switch.ReconfigureDvs_Task(c_spec)
            try:
                wait_for_task(task)
            except Exception:
                self.restore_original_state()
                self.module.fail_json(msg=task.info.error.msg)

    def remove_vspan_session(self):
        """Calls the necessary functions to delete a VSpanSession."""
        results = dict(changed=False, result="")
        mirror_session = self.find_session_by_name()
        if mirror_session is None:
            results['result'] = "There is no VSpanSession with the name: {0:s}.".format(self.name)
            return results
        promiscuous_ports = self.turn_off_promiscuous()
        session_key = mirror_session.key
        # Delete Mirroring Session
        self.delete_mirroring_session(session_key)
        # Session
        self.deleted_session = mirror_session
        # Set back the promiscuous ports
        if promiscuous_ports:
            self.set_port_security_promiscuous(promiscuous_ports, True)
        results['changed'] = True
        results['result'] = 'VSpan Session has been deleted'
        return results

    def check_if_session_name_is_free(self):
        """Checks whether the name is used or not
        Returns
        -------
        bool
            True if the name is free and False if it is used.
        """
        for vspan_session in self.dv_switch.config.vspanSession:
            if vspan_session.name == self.name:
                return False
        return True

    def create_vspan_session(self):
        """Builds up the session, adds the parameters that we specified, then creates it on the vSwitch"""

        session = vim.dvs.VmwareDistributedVirtualSwitch.VspanSession(
            name=self.name,
            enabled=True
        )
        if self.session_type is not None:
            session.sessionType = self.session_type
            if self.session_type == 'encapsulatedRemoteMirrorSource':
                if self.source_port_received is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(portKey=str(self.source_port_received))
                    if not self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=port.portKey)):
                        self.module.fail_json(msg="Couldn't find port: {0:s}".format(self.source_port_received))
                    session.sourcePortReceived = port
                if self.source_port_transmitted is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(portKey=str(self.source_port_transmitted))
                    if not self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=port.portKey)):
                        self.module.fail_json(msg="Couldn't find port: {0:s}".format(self.source_port_transmitted))
                    session.sourcePortTransmitted = port
                if self.destination_port is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(ipAddress=str(self.destination_port))
                    session.destinationPort = port
            if self.session_type == 'remoteMirrorSource':
                if self.source_port_received is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(portKey=str(self.source_port_received))
                    if not self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=port.portKey)):
                        self.module.fail_json(msg="Couldn't find port: {0:s}".format(self.source_port_received))
                    session.sourcePortReceived = port
                if self.source_port_transmitted is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(portKey=str(self.source_port_transmitted))
                    if not self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=port.portKey)):
                        self.module.fail_json(msg="Couldn't find port: {0:s}".format(self.source_port_transmitted))
                    session.sourcePortTransmitted = port
                if self.destination_port is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(uplinkPortName=str(self.destination_port))
                    session.destinationPort = port
            if self.session_type == 'remoteMirrorDest':
                if self.source_port_received is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(vlans=[int(self.source_port_received)])
                    if int(self.source_port_received) not in self.dv_switch.QueryUsedVlanIdInDvs():
                        self.module.fail_json(msg="Couldn't find vlan: {0:s}".format(self.source_port_received))
                    session.sourcePortReceived = port
                if self.destination_port is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(portKey=str(self.destination_port))
                    if not self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=port.portKey)):
                        self.module.fail_json(msg="Couldn't find port: {0:s}".format(self.destination_port))
                    session.destinationPort = port
            if self.session_type == 'dvPortMirror':
                if self.source_port_received is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(portKey=str(self.source_port_received))
                    if not self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=port.portKey)):
                        self.module.fail_json(msg="Couldn't find port: {0:s}".format(self.source_port_received))
                    session.sourcePortReceived = port
                if self.source_port_transmitted is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(portKey=str(self.source_port_transmitted))
                    if not self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=port.portKey)):
                        self.module.fail_json(msg="Couldn't find port: {0:s}".format(self.source_port_transmitted))
                    session.sourcePortTransmitted = port
                if self.destination_port is not None:
                    port = vim.dvs.VmwareDistributedVirtualSwitch.VspanPorts(portKey=str(self.destination_port))
                    if not self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=port.portKey)):
                        self.module.fail_json(msg="Couldn't find port: {0:s}".format(self.destination_port))
                    session.destinationPort = port
        if self.description is not None:
            session.description = self.description
        if self.encapsulation_vlan_id is not None:
            session.encapsulationVlanId = self.encapsulation_vlan_id
        if self.strip_original_vlan is not None:
            session.stripOriginalVlan = self.strip_original_vlan
        if self.mirrored_packet_length is not None:
            session.mirroredPacketLength = self.mirrored_packet_length
        if self.normal_traffic_allowed is not None:
            session.normalTrafficAllowed = self.normal_traffic_allowed
        if self.sampling_rate is not None:
            session.samplingRate = self.sampling_rate
        config_version = self.dv_switch.config.configVersion
        s_spec = vim.dvs.VmwareDistributedVirtualSwitch.VspanConfigSpec(vspanSession=session, operation="add")
        c_spec = vim.dvs.VmwareDistributedVirtualSwitch.ConfigSpec(vspanConfigSpec=[s_spec], configVersion=config_version)
        task = self.dv_switch.ReconfigureDvs_Task(c_spec)
        try:
            wait_for_task(task)
        except Exception:
            self.restore_original_state()
            self.module.fail_json(msg=task.info.error.msg)

    def add_vspan_session(self):
        """Calls the necessary functions to create a VSpanSession"""
        results = dict(changed=False, result="")
        promiscous_ports = self.turn_off_promiscuous()
        if not self.check_if_session_name_is_free():
            self.module.fail_json(msg="There is another VSpan Session with the name: {0:s}.".format(self.name))
        # Locate the ports, we want to use
        dv_ports = None
        ports = [str(self.source_port_received), str(self.source_port_transmitted), str(self.destination_port)]
        if ports:
            dv_ports = self.dv_switch.FetchDVPorts(vim.dvs.PortCriteria(portKey=ports))
        for dv_port in dv_ports:
            if dv_port.config.setting.securityPolicy.allowPromiscuous.value:
                self.set_port_security_promiscuous([dv_port.key], False)
                self.modified_ports.update({dv_port.key: True})
        # Now we can create the VspanSession
        self.create_vspan_session()
        # Finally we can set the destination port to promiscuous mode
        if self.session_type == 'dvPortMirror' or self.session_type == 'remoteMirrorDest':
            self.set_port_security_promiscuous([str(self.destination_port)], True)
        # Set Back the Promiscuous ports
        if promiscous_ports:
            self.set_port_security_promiscuous(promiscous_ports, True)
        results['changed'] = True
        results['result'] = 'Mirroring session has been created.'
        return results


def main():
    argument_spec = vmware_argument_spec()
    argument_spec.update(dict(
        switch=dict(type='str', required=True, aliases=['switch_name']),
        name=dict(type='str', required=True),
        state=dict(type='str', required=True, choices=['present', 'absent']),
        session_type=dict(type='str', default='dvPortMirror', choices=['dvPortMirror',
                                                                       'encapsulatedRemoteMirrorSource',
                                                                       'remoteMirrorDest',
                                                                       'remoteMirrorSource']),
        enabled=dict(type='bool', default=True),
        description=dict(type='str'),
        source_port_transmitted=dict(type='str'),
        source_port_received=dict(type='str'),
        destination_port=dict(type='str'),
        encapsulation_vlan_id=dict(type='int'),
        strip_original_vlan=dict(type='bool'),
        mirrored_packet_length=dict(type='int'),
        normal_traffic_allowed=dict(type='bool'),
        sampling_rate=dict(type='int'),
        source_vm_transmitted=dict(type='dict',
                                   options=dict(
                                       name=dict(type='str'),
                                       nic_label=dict(type='str'))),
        source_vm_received=dict(type='dict',
                                options=dict(
                                    name=dict(type='str'),
                                    nic_label=dict(type='str'))),
        destination_vm=dict(type='dict',
                            options=dict(
                                name=dict(type='str'),
                                nic_label=dict(type='str'))),
    ))
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=False)
    session = VMwareVspanSession(module)
    session.process_operation()


if __name__ == '__main__':
    main()
